@@ -1,7 +1,7 @@
 /*
  * module.h	Interface to the RADIUS module system.
  *
- * Version:	$Id: modules.h,v 1.20 2002/10/21 15:41:06 aland Exp $
+ * Version:	$Id: modules.h,v 1.22 2003/07/04 19:11:07 aland Exp $
  *
  */
 
@@ -56,13 +56,10 @@ int detach_modules(void);
 int module_authorize(int type, REQUEST *request);
 int module_authenticate(int type, REQUEST *request);
 int module_preacct(REQUEST *request);
-int module_accounting(REQUEST *request);
-int module_checksimul(REQUEST *request, int maxsimul);
+int module_accounting(int type, REQUEST *request);
+int module_checksimul(int type, REQUEST *request, int maxsimul);
 int module_pre_proxy(REQUEST *request);
 int module_post_proxy(REQUEST *request);
-int module_post_auth(REQUEST *request);
-int module_pre_proxy(REQUEST *request);
-int module_post_proxy(REQUEST *request);
-int module_post_auth(REQUEST *request);
+int module_post_auth(int type, REQUEST *request);
 
 #endif /* RADIUS_MODULES_H */
