@@ -1,7 +1,7 @@
 /*
  * radius.h	Constants of the radius protocol.
  *
- * Version:	$Id: radius.h,v 1.46 2002/08/16 14:46:21 kkalev Exp $
+ * Version:	$Id: radius.h,v 1.54 2003/04/21 20:39:57 aland Exp $
  *
  */
 
@@ -26,12 +26,26 @@
 #define PW_ACCESS_CHALLENGE		11
 #define PW_STATUS_SERVER		12
 #define PW_STATUS_CLIENT		13
+#define PW_DISCONNECT_REQUEST		40
+#define PW_DISCONNECT_ACK		41
+#define PW_DISCONNECT_NAK		42
+#define PW_COF_REQUEST			43
+#define PW_COF_ACK			44
+#define PW_COF_NAK			45
+
+#define PW_AUTH_UDP_PORT                1812
+#define PW_ACCT_UDP_PORT                1813
+#define PW_POD_UDP_PORT			1700
 
 #define	PW_USER_NAME			1
 #define	PW_USER_PASSWORD		2
 #define	PW_PASSWORD			2
 #define	PW_CHAP_PASSWORD		3
 #define	PW_NAS_IP_ADDRESS		4
+#define	PW_NAS_PORT			5
+/*
+ *  FIXME: We should remove this...
+ */
 #define	PW_NAS_PORT_ID			5
 #define	PW_SERVICE_TYPE			6
 #define	PW_FRAMED_PROTOCOL		7
@@ -97,7 +111,7 @@
 
 #define PW_ARAP_CHALLENGE_RESPONSE	84
 #define PW_NAS_PORT_ID_STRING  		87
-#define PW_FRAMED_POOL			89
+#define PW_FRAMED_POOL			88
 
 #define PW_DIGEST_RESPONSE		206
 #define PW_DIGEST_ATTRIBUTES		207
@@ -107,7 +121,7 @@
 #define PW_EXEC_PROGRAM			502
 #define PW_EXEC_PROGRAM_WAIT		503
 
-#define PW_AUTHTYPE			1000
+#define PW_AUTH_TYPE			1000
 #define PW_PREFIX			1003
 #define PW_SUFFIX			1004
 #define PW_GROUP			1005
@@ -116,7 +130,14 @@
 #define PW_ADD_PREFIX			1008
 #define PW_ADD_SUFFIX			1009
 #define PW_EXPIRATION			1010
-#define PW_AUTZTYPE			1011
+#define PW_AUTZ_TYPE			1011
+#define PW_ACCT_TYPE			1012
+#define PW_SESSION_TYPE			1013
+#define PW_POST_AUTH_TYPE		1014
+#define PW_PRE_PROXY_TYPE		1015
+#define PW_POST_PROXY_TYPE		1016
+#define PW_PRE_ACCT_TYPE		1017
+
 #define PW_USER_CATEGORY		1029
 #define PW_GROUP_NAME			1030
 #define PW_HUNTGROUP_NAME		1031
@@ -160,6 +181,7 @@
 #define PW_X99_FAST			1077
 #define PW_REWRITE_RULE			1078
 #define PW_SQL_GROUP			1079
+#define PW_RESPONSE_PACKET_TYPE		1080
 
 /*
  *	Integer Translations
@@ -237,3 +259,12 @@
  * Vendor specific attributes
  */
 #define PW_FREERADIUS_PROXIED_TO	((VENDORPEC_FREERADIUS<<16)|1)
+
+/*
+ *  Old nonsense.  Will be deleted ASAP
+ */
+#define PW_AUTHTYPE			1000
+#define PW_AUTZTYPE			1011
+#define PW_ACCTTYPE			1012
+#define PW_SESSTYPE			1013
+#define PW_POSTAUTHTYPE			1014
