@@ -1,7 +1,7 @@
 /*
  * radius.c	Functions to send/receive radius packets.
  *
- * Version:	$Id: radius.c,v 1.99.2.3 2003/09/29 15:26:24 aland Exp $
+ * Version:	$Id: radius.c,v 1.99.2.5 2003/11/20 17:47:09 aland Exp $
  *
  *   This library is free software; you can redistribute it and/or
  *   modify it under the terms of the GNU Lesser General Public
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: radius.c,v 1.99.2.3 2003/09/29 15:26:24 aland Exp $";
+static const char rcsid[] = "$Id: radius.c,v 1.99.2.5 2003/11/20 17:47:09 aland Exp $";
 
 #include	"autoconf.h"
 #include	"md5.h"
@@ -1291,7 +1291,7 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original,
 			    pair->type == PW_TYPE_STRING) {
 				int offset = 0;
 
-				if(TAG_VALID(*ptr)) {
+				if ((pair->length > 0) && TAG_VALID(*ptr)) {
 					pair->flags.tag = *ptr;
 					pair->length--;
 					offset = 1;
@@ -1305,11 +1305,15 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original,
 					 * otherwise, the Tag field SHOULD be ignored.
 					 */
 					pair->flags.tag = 0x00;
-					pair->length--;
+					if (pair->length > 0) pair->length--;
 					offset = 1;
 				} else {
 				       pair->flags.tag = 0x00;
 				}
+
+				/*
+				 *	pair->length may be zero here...
+				 */
 				memcpy(pair->strvalue, ptr + offset,
 				       pair->length);
 			} else {
@@ -1359,6 +1363,10 @@ int rad_decode(RADIUS_PACKET *packet, RADIUS_PACKET *original,
 				 *  Tunnel-Password
 				 */
 			case FLAG_ENCRYPT_TUNNEL_PASSWORD:
+				if (!original) {
+					librad_log("ERROR: Tunnel-Password attribute in request: Cannot decrypt it.");
+					return -1;
+				}
 			        rad_tunnel_pwdecode((char *)pair->strvalue,
 						    &pair->length, 
 						    secret,
