@@ -2,8 +2,23 @@
  * token.c	Read the next token from a string.
  *		Yes it's pretty primitive but effective.
  *
- * Version:	$Id: token.c,v 1.15 2002/07/10 18:01:06 aland Exp $
+ * Version:	$Id: token.c,v 1.16.2.1 2003/09/16 18:34:37 phampson Exp $
  *
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
+ *
+ *   This library is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ *   Lesser General Public License for more details.
+ *
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with this library; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
+ *
+ * Copyright 2000  The FreeRADIUS server project
  */
 
 #include <stdio.h>
@@ -12,14 +27,9 @@
 #include <ctype.h>
 #include "token.h"
 
-static const char rcsid[] = "$Id: token.c,v 1.15 2002/07/10 18:01:06 aland Exp $";
+static const char rcsid[] = "$Id: token.c,v 1.16.2.1 2003/09/16 18:34:37 phampson Exp $";
 
-typedef struct {
-	const char *str;
-	int token;
-} TOKEN;
-
-static const TOKEN tokens[] = {
+static const LRAD_NAME_NUMBER tokens[] = {
 	{ "=~", T_OP_REG_EQ,	}, /* order is important! */
 	{ "!~", T_OP_REG_NE,	},
 	{ "{",	T_LCBRACE,	},
@@ -60,13 +70,13 @@ static const TOKEN tokens[] = {
  *	Returns 0 or special token value.
  */
 static LRAD_TOKEN getthing(char **ptr, char *buf, int buflen, int tok,
-			   const TOKEN *tokenlist)
+			   const LRAD_NAME_NUMBER *tokenlist)
 {
 	char	*s, *p;
 	int	quote;
 	int	escape;
 	int	x;
-	const TOKEN	*t;
+	const LRAD_NAME_NUMBER*t;
 	LRAD_TOKEN rcode;
 
 	buf[0] = 0;
@@ -84,14 +94,14 @@ static LRAD_TOKEN getthing(char **ptr, char *buf, int buflen, int tok,
 	/*
 	 *	Might be a 1 or 2 character token.
 	 */
-	if (tok) for (t = tokenlist; t->str; t++) {
-		if (TOKEN_MATCH(p, t->str)) {
-			strcpy(buf, t->str);
-			p += strlen(t->str);
+	if (tok) for (t = tokenlist; t->name; t++) {
+		if (TOKEN_MATCH(p, t->name)) {
+			strcpy(buf, t->name);
+			p += strlen(t->name);
 			while (isspace((int) *p))
 				p++;
 			*ptr = p;
-			return t->token;
+			return (LRAD_TOKEN) t->number;
 		}
 	}
 
@@ -153,10 +163,10 @@ static LRAD_TOKEN getthing(char **ptr, char *buf, int buflen, int tok,
 			if (isspace((int) *p))
 				break;
 			if (tok) {
-				for (t = tokenlist; t->str; t++)
-					if (TOKEN_MATCH(p, t->str))
+				for (t = tokenlist; t->name; t++)
+					if (TOKEN_MATCH(p, t->name))
 						break;
-				if (t->str != NULL)
+				if (t->name != NULL)
 					break;
 			}
 		}
@@ -223,3 +233,36 @@ LRAD_TOKEN gettoken(char **ptr, char *buf, int buflen)
 {
 	return getthing(ptr, buf, buflen, 1, tokens);
 }
+
+/*
+ *	Convert a string to an integer
+ */
+int lrad_str2int(const LRAD_NAME_NUMBER *table, const char *name, int def)
+{
+	const LRAD_NAME_NUMBER *this;
+
+	for (this = table; this->name != NULL; this++) {
+		if (strcasecmp(this->name, name) == 0) {
+			return this->number;
+		}
+	}
+
+	return def;
+}
+
+/*
+ *	Convert an integer to a string.
+ */
+const char *lrad_int2str(const LRAD_NAME_NUMBER *table, int number,
+			 const char *def)
+{
+	const LRAD_NAME_NUMBER *this;
+
+	for (this = table; this->name != NULL; this++) {
+		if (this->number == number) {
+			return this->name;
+		}
+	}
+
+	return def;
+}
