@@ -1,11 +1,26 @@
 /*
  * valuepair.c	Functions to handle VALUE_PAIRs
  *
- * Version:	$Id: valuepair.c,v 1.57 2002/08/28 19:24:00 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.62.2.2 2003/10/09 00:47:14 phampson Exp $
  *
+ *   This library is free software; you can redistribute it and/or
+ *   modify it under the terms of the GNU Lesser General Public
+ *   License as published by the Free Software Foundation; either
+ *   version 2.1 of the License, or (at your option) any later version.
+ *
+ *   This library is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
+ *   Lesser General Public License for more details.
+ *
+ *   You should have received a copy of the GNU Lesser General Public
+ *   License along with this library; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA
+ *
+ * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.57 2002/08/28 19:24:00 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.62.2.2 2003/10/09 00:47:14 phampson Exp $";
 
 #include	"autoconf.h"
 
@@ -49,10 +64,20 @@ VALUE_PAIR *paircreate(int attr, int type)
 	vp->attribute = attr;
 	vp->operator = T_OP_EQ;
 	vp->type = type;
-	if ((da = dict_attrbyvalue(attr)) != NULL)
+
+	/*
+	 *	Dictionary type over-rides what the caller says.
+	 */
+	if ((da = dict_attrbyvalue(attr)) != NULL) {
 		strcpy(vp->name, da->name);
-	else
-		sprintf(vp->name, "Attr-%d", attr);
+		vp->type = da->type;
+		vp->flags = da->flags;
+	} else if (VENDOR(attr) == 0) {
+		sprintf(vp->name, "Attr-%u", attr);
+	} else {
+		sprintf(vp->name, "Vendor-%u-Attr-%u",
+			VENDOR(attr), attr & 0xffff);
+	}
 	switch (vp->type) {
 		case PW_TYPE_INTEGER:
 		case PW_TYPE_IPADDR:
@@ -299,6 +324,11 @@ void pairmove(VALUE_PAIR **to, VALUE_PAIR **from)
 			  break;
 #endif
 			case T_OP_EQ:		/* = */
+				/*
+				 *  FIXME: Tunnel attributes with
+				 *  different tags are different
+				 *  attributes.
+				 */
 				if (found) {
 					tailfrom = i;
 					continue; /* with the loop */
@@ -579,7 +609,7 @@ VALUE_PAIR *pairparsevalue(VALUE_PAIR *vp, const char *value)
 			 *	Note that ALL integers are unsigned!
 			 */
 			if (isdigit((int) *value)) {
-				vp->lvalue = atoi(value);
+				vp->lvalue = (uint32_t) strtoul(value, NULL, 10);
 				vp->length = 4;
 			}
 			/*
@@ -666,6 +696,10 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 	char            *tc, *ts;
 	signed char     tag;
 	int             found_tag;
+#ifdef HAVE_REGEX_H
+	int		res;
+	regex_t		cre;
+#endif
 
 	/*
 	 *    Check for tags in 'Attribute:Tag' format.
@@ -680,7 +714,7 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 		         /* Wildcard tag for check items */
 		         tag = TAG_ANY;
 			 *ts = 0;
-		 } else if ((ts[1] >= 0) && (ts[1] <= '9')) {
+		 } else if ((ts[1] >= '0') && (ts[1] <= '9')) {
 		         /* It's not a wild card tag */
 		         tag = strtol(ts + 1, &tc, 0);
 			 if (tc && !*tc && TAG_VALID_ZERO(tag))
@@ -761,7 +795,6 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 	        return vp;
 		break;
 
-#ifdef HAVE_REGEX_H
 		/*
 		 *	Regular expression comparison of integer attributes
 		 *	does a STRING comparison of the names of their
@@ -772,6 +805,32 @@ VALUE_PAIR *pairmake(const char *attribute, const char *value, int operator)
 		if (vp->type == PW_TYPE_INTEGER) {
 			return vp;
 		}
+#ifdef HAVE_REGEX_H
+		/*
+		 *	Regular expression match with no regular
+		 *	expression is wrong.
+		 */
+		if (!value) {
+			pairfree(&vp);
+			return NULL;
+		}
+
+		res = regcomp(&cre, value, REG_EXTENDED|REG_NOSUB);
+		if (res != 0) {
+			char	msg[128];
+
+			regerror(res, &cre, msg, sizeof(msg));               
+			librad_log("Illegal regular expression in attribute: %s: %s",
+				vp->name, msg);
+			free(vp);
+			return NULL;
+		}
+		regfree(&cre);
+#else
+		librad_log("Regelar expressions not enabled in this build, error in attribute %s",
+				vp->name);
+		free(vp);
+		return NULL;
 #endif
 	}
 
