@@ -1,7 +1,7 @@
 /*
  * auth.c	User authentication.
  *
- * Version:	$Id: auth.c,v 1.112 2002/10/08 21:48:27 aland Exp $
+ * Version:	$Id: auth.c,v 1.127 2003/06/24 14:22:19 pnixon Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  */
 
-static const char rcsid[] = "$Id: auth.c,v 1.112 2002/10/08 21:48:27 aland Exp $";
+static const char rcsid[] = "$Id: auth.c,v 1.127 2003/06/24 14:22:19 pnixon Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -31,10 +31,6 @@ static const char rcsid[] = "$Id: auth.c,v 1.112 2002/10/08 21:48:27 aland Exp $
 #include <string.h>
 #include <ctype.h>
 
-#if HAVE_CRYPT_H
-#	include <crypt.h>
-#endif
-
 #if HAVE_NETINET_IN_H
 #	include <netinet/in.h>
 #endif
@@ -53,10 +49,10 @@ char *auth_name(char *buf, size_t buflen, REQUEST *request, int do_cli) {
 
 	if ((cli = pairfind(request->packet->vps, PW_CALLING_STATION_ID)) == NULL)
 		do_cli = 0;
-	if ((pair = pairfind(request->packet->vps, PW_NAS_PORT_ID)) != NULL)
+	if ((pair = pairfind(request->packet->vps, PW_NAS_PORT)) != NULL)
 		port = pair->lvalue;
 
-	snprintf(buf, buflen, "from client %.128s port %d%s%.128s",
+	snprintf(buf, buflen, "from client %.128s port %u%s%.128s",
 			client_name(request->packet->src_ipaddr), port,
 			(do_cli ? " cli " : ""), (do_cli ? (char *)cli->strvalue : ""));
 
@@ -173,27 +169,6 @@ static int rad_authlog(const char *msg, REQUEST *request, int goodpass) {
 	return 0;
 }
 
-/*
- *       returncheck - Check for Auth-Type = Reject and return appropriate
- *                     module return code if it is found.
- */
-int rad_check_return(VALUE_PAIR *list)
-{
-	VALUE_PAIR	*authtype;
-
-	/* 
-	 * We check for Auth-Type = Reject here
-	 */
-
-	authtype = pairfind(list, PW_AUTHTYPE);
-	if((authtype) && authtype->lvalue == PW_AUTHTYPE_REJECT)  {
-		DEBUG2("rad_check_return:  Auth-Type is Reject");
-		return RLM_MODULE_REJECT;
-	}
-
-	return RLM_MODULE_UPDATED;
-}
-
 /*
  *	Check password.
  *
@@ -211,7 +186,6 @@ int rad_check_password(REQUEST *request)
 	VALUE_PAIR *password_pair;
 	VALUE_PAIR *auth_item;
 	char string[MAX_STRING_LEN];
-	const char *crypted_password;
 	int auth_type = -1;
 	int result;
 	int auth_type_count = 0;
@@ -223,7 +197,7 @@ int rad_check_password(REQUEST *request)
 	 *	PW_AUTHTYPE_REJECT.
 	 */
 	cur_config_item = request->config_items;
-	while(((auth_type_pair = pairfind(cur_config_item, PW_AUTHTYPE))) != NULL) {
+	while(((auth_type_pair = pairfind(cur_config_item, PW_AUTH_TYPE))) != NULL) {
 		auth_type = auth_type_pair->lvalue;
 		auth_type_count++;
 
@@ -297,16 +271,13 @@ int rad_check_password(REQUEST *request)
 				return -1;
 			}
 					
-			crypted_password = crypt((char *)auth_item->strvalue,
-						 (char *)password_pair->strvalue);
-			if (!crypted_password) {
-				rad_authlog("Login incorrect "
-					    "(system failed to supply an encrypted password for comparison)", request, 0);
-				return -1;
-			}
-			if (strcmp((char *)password_pair->strvalue,
-				   crypted_password) != 0) {
-				return -1;
+			switch (lrad_crypt_check((char *)auth_item->strvalue,
+									 (char *)password_pair->strvalue)) {
+			case -1:
+			  rad_authlog("Login incorrect "
+						  "(system failed to supply an encrypted password for comparison)", request, 0);
+			case 1:
+			  return -1;
 			}
 			break;
 		case PW_AUTHTYPE_LOCAL:
@@ -339,6 +310,7 @@ int rad_check_password(REQUEST *request)
 			if (auth_item->attribute == PW_PASSWORD) {
 				if (strcmp((char *)password_pair->strvalue,
 					   (char *)auth_item->strvalue) != 0) {
+					DEBUG2("auth: user supplied User-Password does NOT match local User-Password");
 					return -1;
 				}
 				DEBUG2("auth: user supplied User-Password matches local User-Password");
@@ -358,13 +330,15 @@ int rad_check_password(REQUEST *request)
 			 *	Compare them
 			 */
 			if (memcmp(string + 1, auth_item->strvalue + 1,
-					CHAP_VALUE_LENGTH) != 0)
-				result = -1;
+				   CHAP_VALUE_LENGTH) != 0) {
+				DEBUG2("auth: user supplied CHAP-Password does NOT match local User-Password");
+				return -1;
+			}
 			DEBUG2("auth: user supplied CHAP-Password matches local User-Password");
 			break;
 		default:
 			DEBUG2("auth: type \"%s\"",
-					dict_valbyattr(PW_AUTHTYPE, auth_type)->name);
+					dict_valbyattr(PW_AUTH_TYPE, auth_type)->name);
 			/*
 			 *	See if there is a module that handles
 			 *	this type, and turn the RLM_ return
@@ -416,6 +390,7 @@ int rad_authenticate(REQUEST *request)
 	VALUE_PAIR	*module_msg;
 	VALUE_PAIR	*tmp = NULL;
 	VALUE_PAIR	*autz_type_item = NULL;
+	VALUE_PAIR	*postauth_type_item = NULL;
 	int		result, r;
 	char		umsg[MAX_STRING_LEN + 1];
 	const char	*user_msg = NULL;
@@ -426,18 +401,10 @@ int rad_authenticate(REQUEST *request)
 	char		buf[1024], logstr[1024];
 	char		autz_retry = 0;
 	int		autz_type = 0;
+	int		postauth_type = 0;
 
 	password = "";
 
-	/*
-	 *	Free any pre-existing configuration items.
-	 *
-	 *	This should ONLY be happening for proxy replies.
-	 */
-	if ((request->proxy_reply) && (request->config_items)) {
-		pairfree(&request->config_items);
-	}
-
 	/*
 	 *	If this request got proxied to another server,
 	 *	AND it was an authentication request, then we need
@@ -447,39 +414,31 @@ int rad_authenticate(REQUEST *request)
 	 */
 	if ((request->proxy_reply) &&
 	    (request->packet->code == PW_AUTHENTICATION_REQUEST)) {
-		tmp = paircreate(PW_AUTHTYPE, PW_TYPE_INTEGER);
+		tmp = paircreate(PW_AUTH_TYPE, PW_TYPE_INTEGER);
 		if (tmp == NULL) {
 			radlog(L_ERR|L_CONS, "no memory");
 			exit(1);
 		}
 
+		/*
+		 *	Challenges are punted back to the NAS
+		 *	without any further processing.
+		 */
+		if (request->proxy_reply->code == PW_ACCESS_CHALLENGE) {
+			request->reply->code = PW_ACCESS_CHALLENGE;
+			return RLM_MODULE_HANDLED;
+		}
+
 		/*
 		 *	Reply of ACCEPT means accept, ALL other
 		 *	replies mean reject.  This is fail-safe.
 		 */
-		if ((request->proxy_reply->code == PW_AUTHENTICATION_ACK) ||
-		    (request->proxy_reply->code == PW_ACCESS_CHALLENGE))
+		if (request->proxy_reply->code == PW_AUTHENTICATION_ACK)
 			tmp->lvalue = PW_AUTHTYPE_ACCEPT;
 		else
 			tmp->lvalue = PW_AUTHTYPE_REJECT;
 		pairadd(&request->config_items, tmp);
 
-		/*
-		 *	Initialize our reply to the user, by taking
-		 *	the reply attributes from the proxy.
-		 *
-		 *	Note that we DELETE the Proxy-State attributes
-		 *	from the proxy reply, as they include the one
-		 *	we added, which MUST NOT go back to the NAS.
-		 */
-		if (request->proxy_reply->vps) {
-			pairfree(&request->reply->vps);
-			request->reply->vps = request->proxy_reply->vps;
-			pairdelete(&request->reply->vps, PW_PROXY_STATE);
-			request->reply->code = request->proxy_reply->code;
-			request->proxy_reply->vps = NULL;
-		}
-
 		/*
 		 *	If it's an Access-Reject, then do NOT do any
 		 *	authorization or authentication.  They're being
@@ -502,17 +461,25 @@ int rad_authenticate(REQUEST *request)
 	 */
 	namepair = request->username;
 
+	/*
+	 *	Look for, and cache, passwords.
+	 */
+	if (!request->password) {
+		request->password = pairfind(request->packet->vps,
+					     PW_PASSWORD);
+	}
+	  
 	/*
 	 *	Discover which password we want to use.
 	 */
-	if ((auth_item = rad_getpass(request)) != NULL) {
+	auth_item = request->password;
+	if (auth_item) {
 		password = (const char *)auth_item->strvalue;
-	}
 
-	/*
-	 *	Maybe there's a CHAP-Password?
-	 */
-	if (auth_item == NULL) {
+	} else {
+		/*
+		 *	Maybe there's a CHAP-Password?
+		 */
 		if ((auth_item = pairfind(request->packet->vps, 
 				PW_CHAP_PASSWORD)) != NULL) {
 			password = "<CHAP-PASSWORD>";
@@ -532,9 +499,9 @@ int rad_authenticate(REQUEST *request)
 autz_redo:
 	r = module_authorize(autz_type, request);
 	if (r != RLM_MODULE_NOTFOUND &&
-			r != RLM_MODULE_NOOP &&
-			r != RLM_MODULE_OK &&
-			r != RLM_MODULE_UPDATED) {
+	    r != RLM_MODULE_NOOP &&
+	    r != RLM_MODULE_OK &&
+	    r != RLM_MODULE_UPDATED) {
 		if (r != RLM_MODULE_FAIL && r != RLM_MODULE_HANDLED) {
 			if ((module_msg = pairfind(request->packet->vps,
 					PW_MODULE_FAILURE_MESSAGE)) != NULL){
@@ -555,7 +522,7 @@ autz_redo:
 		return r;
 	}
 	if (!autz_retry){
-		autz_type_item = pairfind(request->config_items, PW_AUTZTYPE);
+		autz_type_item = pairfind(request->config_items, PW_AUTZ_TYPE);
 		if (autz_type_item){
 			autz_type = autz_type_item->lvalue;
 			autz_retry = 1;
@@ -632,12 +599,19 @@ autz_redo:
 
 	if (result >= 0 &&
 			(check_item = pairfind(request->config_items, PW_SIMULTANEOUS_USE)) != NULL) {
+		VALUE_PAIR	*session_type;
+		int		sess_type = 0;
+
+		session_type = pairfind(request->config_items, PW_SESSION_TYPE);
+		if (session_type)
+			sess_type = session_type->lvalue;
+
 		/*
 		 *	User authenticated O.K. Now we have to check
 		 *	for the Simultaneous-Use parameter.
 		 */
 		if (namepair &&
-				(r = module_checksimul(request, check_item->lvalue)) != 0) {
+		    (r = module_checksimul(sess_type,request, check_item->lvalue)) != 0) {
 			char mpp_ok = 0;
 
 			if (r == 2){
@@ -762,9 +736,10 @@ autz_redo:
 		 *  Find the NAS port ID.
 		 */
 		if ((vpPortId = pairfind(request->packet->vps,
-					 PW_NAS_PORT_ID)) != NULL) {
+					 PW_NAS_PORT)) != NULL) {
 		  tmp->lvalue = htonl(ntohl(tmp->lvalue) + vpPortId->lvalue);
 		  tmp->flags.addport = 0;
+		  ip_ntoa(tmp->strvalue, tmp->lvalue);
 		} else {
 			DEBUG2("WARNING: No NAS-Port attribute in request.  CANNOT return a Framed-IP-Address + NAS-Port.\n");
 			pairdelete(&request->reply->vps, PW_FRAMED_IP_ADDRESS);
@@ -810,17 +785,30 @@ autz_redo:
 	 *	do it first before sending the reply.
 	 */
 	if (exec_program && exec_wait) {
-		if (radius_exec_program(exec_program, request,
-				exec_wait, &user_msg) != 0) {
-			free(exec_program);
+		r = radius_exec_program(exec_program, request,
+					exec_wait,
+					umsg, sizeof(umsg),
+					request->packet->vps, &tmp);
+		free(exec_program);
+		exec_program = NULL;
 
+		/*
+		 *	Always add the value-pairs to the reply.
+		 */
+		pairmove(&request->reply->vps, &tmp);
+		pairfree(&tmp);
+
+		if (r != 0) {
 			/*
 			 *	Error. radius_exec_program() returns -1 on
 			 *	fork/exec errors, or >0 if the exec'ed program
 			 *	had a non-zero exit status.
 			 */
-			if (user_msg == NULL)
+			if (umsg[0] == '\0') {
 				user_msg = "\r\nAccess denied (external check failed).";
+			} else {
+				user_msg = &umsg[0];
+			}
 
 			request->reply->code = PW_AUTHENTICATION_REJECT;
 			tmp = pairmake("Reply-Message", user_msg, T_OP_SET);
@@ -829,7 +817,7 @@ autz_redo:
 			rad_authlog("Login incorrect (external check failed)", 
 					request, 0);
 
-			return RLM_MODULE_OK;
+			return RLM_MODULE_REJECT;
 		}
 	}
 
@@ -893,7 +881,8 @@ autz_redo:
 		/*
 		 *	No need to check the exit status here.
 		 */
-		radius_exec_program(exec_program, request, exec_wait, NULL);
+		radius_exec_program(exec_program, request, exec_wait,
+				    NULL, 0, request->packet->vps, NULL);
 	}
 
 	if (exec_program) 
@@ -903,8 +892,12 @@ autz_redo:
 	 *	Do post-authentication calls. ignoring the return code.
 	 *	If the post-authentication
 	 */
-	result = module_post_auth(request);
+	postauth_type_item = pairfind(request->config_items, PW_POST_AUTH_TYPE);
+	if (postauth_type_item)
+		postauth_type = postauth_type_item->lvalue;
+	result = module_post_auth(postauth_type, request);
 	switch (result) {
+	default:
 	  break;
 	  
 	  /*
@@ -932,43 +925,3 @@ autz_redo:
 
 	return result;
 }
-
-
-/*
- * Find the password pair, decode pass if
- * needed, and return the value pair.  If
- * not found, return NULL
- */
-VALUE_PAIR *rad_getpass(REQUEST *request) {
-	VALUE_PAIR *auth_item;
-
-	/*
-	 *	First, look up the password in the request header.
-	 */
-	auth_item = request->password;
-	if (auth_item) {
-		/*
-		 *	It's there, but it's not a clear-text password.
-		 *	Give up.
-		 */
-		if (auth_item->attribute != PW_PASSWORD) {
-			return NULL;
-		}
-	} else {
-		/*
-		 *	Go find the request password.
-		 */
-		auth_item = pairfind(request->packet->vps, PW_PASSWORD);
-		if (!auth_item) {
-			return NULL;
-		}
-
-		/*
-		 *	Save the found password for later.
-		 */
-		request->password = auth_item;
-	}
-
-	return auth_item;
-}
-
