@@ -5,7 +5,7 @@
  *		write a decent parser. I know how to do that, really :)
  *		miquels@cistron.nl
  *
- * Version:	$Id: conffile.c,v 1.90 2002/10/30 19:52:55 aland Exp $
+ * Version:	$Id: conffile.c,v 1.93.2.1 2003/07/09 15:55:22 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -43,7 +43,7 @@
 #include "modules.h"
 
 static const char rcsid[] =
-"$Id: conffile.c,v 1.90 2002/10/30 19:52:55 aland Exp $";
+"$Id: conffile.c,v 1.93.2.1 2003/07/09 15:55:22 aland Exp $";
 
 #define xstrdup strdup
 
@@ -328,7 +328,8 @@ static const char *cf_expand_variables(const char *cf, int *lineno,
 /*
  *	Parse a configuration section into user-supplied variables.
  */
-int cf_section_parse(CONF_SECTION *cs, void *base, const CONF_PARSER *variables)
+int cf_section_parse(CONF_SECTION *cs, void *base,
+		     const CONF_PARSER *variables)
 {
 	int i;
 	int rcode;
@@ -410,7 +411,7 @@ int cf_section_parse(CONF_SECTION *cs, void *base, const CONF_PARSER *variables)
 			
 		case PW_TYPE_STRING_PTR:
 			q = (char **) data;
-			if (base == NULL && *q != NULL) {
+			if (*q != NULL) {
 				free(*q);
 			}
 
@@ -420,7 +421,9 @@ int cf_section_parse(CONF_SECTION *cs, void *base, const CONF_PARSER *variables)
 			 *	been expanded.
 			 */
 			if (value && (value == variables[i].dflt)) {
-				value = cf_expand_variables(NULL, 0, cs, buffer,value);
+				value = cf_expand_variables("?",
+							    &cs->item.lineno,
+							    cs, buffer, value);
 				if (!value) {
 					return -1;
 				}
@@ -479,6 +482,8 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 	char buf2[8192];
 	char buf3[8192];
 	int t1, t2, t3;
+	char *cbuf = buf;
+	int len;
 	
 	/*
 	 *	Ensure that the user can't add CONF_SECTIONs
@@ -497,12 +502,54 @@ static CONF_SECTION *cf_section_read(const char *cf, int *lineno, FILE *fp,
 	cs->item.lineno = *lineno;
 
 	/*
-	 *	Read.
+	 *	Read, checking for line continuations ('\\' at EOL)
 	 */
-	while (fgets(buf, sizeof(buf), fp) != NULL) {
-		(*lineno)++;
-		ptr = buf;
+	for (;;) {
+		int eof;
 
+		/*
+		 *	Get data, and remember if we are at EOF.
+		 */
+		eof = (fgets(cbuf, sizeof(buf) - (cbuf - buf), fp) == NULL);
+		(*lineno)++;
+
+		len = strlen(cbuf);
+
+		/*
+		 *	We've filled the buffer, and there isn't
+		 *	a CR in it.  Die!
+		 */
+		if ((len == sizeof(buf)) &&
+		    (cbuf[len - 1] != '\n')) {
+			radlog(L_ERR, "%s[%d]: Line too long",
+			       cf, *lineno);
+			cf_section_free(&cs);
+			return NULL;
+		}
+
+		/*
+		 *  Check for continuations.
+		 */
+		if (cbuf[len - 1] == '\n') len--;
+
+		/*
+		 *	Last character is '\\'.  Over-write it,
+		 *	and read another line.
+		 */
+		if ((len > 0) && (cbuf[len - 1] == '\\')) {
+			cbuf[len - 1] = '\0';
+			cbuf += len - 1;
+			continue;
+		}
+
+		/*
+		 *  We're at EOF, and haven't read anything.  Stop.
+		 */
+		if (eof && (cbuf == buf)) {
+			break;
+		}
+
+		ptr = cbuf = buf;
 		t1 = gettoken(&ptr, buf1, sizeof(buf1));
 
 		/*
