@@ -1,7 +1,7 @@
 /*
  * modcall.c
  *
- * Version:	$Id: modcall.c,v 1.14 2002/06/14 15:17:38 aland Exp $
+ * Version:	$Id: modcall.c,v 1.15.2.1 2003/09/28 13:51:25 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -97,46 +97,33 @@ static void add_child(modgroup *g, modcallable *c)
 
 /* Here's where we recognize all of our keywords: first the rcodes, then the
  * actions */
+static LRAD_NAME_NUMBER rcode_table[] = {
+	{ "reject",     RLM_MODULE_REJECT       },
+	{ "fail",       RLM_MODULE_FAIL         },
+	{ "ok",         RLM_MODULE_OK           },
+	{ "handled",    RLM_MODULE_HANDLED      },
+	{ "invalid",    RLM_MODULE_INVALID      },
+	{ "userlock",   RLM_MODULE_USERLOCK     },
+	{ "notfound",   RLM_MODULE_NOTFOUND     },
+	{ "noop",       RLM_MODULE_NOOP         },
+	{ "updated",    RLM_MODULE_UPDATED      },
+	{ NULL, 0 }
+};
+
 static int str2rcode(const char *s, const char *filename, int lineno)
 {
-	if(!strcasecmp(s, "reject"))
-		return RLM_MODULE_REJECT;
-	else if(!strcasecmp(s, "fail"))
-		return RLM_MODULE_FAIL;
-	else if(!strcasecmp(s, "ok"))
-		return RLM_MODULE_OK;
-	else if(!strcasecmp(s, "handled"))
-		return RLM_MODULE_HANDLED;
-	else if(!strcasecmp(s, "invalid"))
-		return RLM_MODULE_INVALID;
-	else if(!strcasecmp(s, "userlock"))
-		return RLM_MODULE_USERLOCK;
-	else if(!strcasecmp(s, "notfound"))
-		return RLM_MODULE_NOTFOUND;
-	else if(!strcasecmp(s, "noop"))
-		return RLM_MODULE_NOOP;
-	else if(!strcasecmp(s, "updated"))
-		return RLM_MODULE_UPDATED;
-	else {
+	int rcode;
+
+	rcode = lrad_str2int(rcode_table, s, -1);
+	if (rcode < 0) {
 		radlog(L_ERR|L_CONS,
-			"%s[%d] Unknown module rcode '%s'.\n",
-			filename, lineno, s);
+		       "%s[%d] Unknown module rcode '%s'.\n",
+		       filename, lineno, s);
 		exit(1);
 	}
-	return -1;
-}
 
-static const char *rcode2str[] = {
-	"reject",
-	"fail",
-	"ok",
-	"handled",
-	"invalid",
-	"userlock",
-	"notfound",
-	"noop",
-	"updated"
-};
+	return rcode;
+}
 
 static int str2action(const char *s, const char *filename, int lineno)
 {
@@ -207,10 +194,16 @@ static int call_modsingle(int component, modsingle *sp, REQUEST *request,
 {
 	int myresult = default_result;
 
+	DEBUG3("  modsingle[%s]: calling %s (%s) for request %d",
+	       comp2str[component], sp->modinst->name, 
+	       sp->modinst->entry->name, request->number);
 	safe_lock(sp->modinst);
 	myresult = sp->modinst->entry->module->methods[component](
 			sp->modinst->insthandle, request);
 	safe_unlock(sp->modinst);
+	DEBUG3("  modsingle[%s]: returned from %s (%s) for request %d",
+	       comp2str[component], sp->modinst->name, 
+	       sp->modinst->entry->name, request->number);
 
 	return myresult;
 }
@@ -234,7 +227,7 @@ static int call_modgroup(int component, modgroup *g, REQUEST *request,
 
 #if 0
 		DEBUG2("%s: action for %s is %s",
-			comp2str[component], rcode2str[r],
+			comp2str[component], lrad_int2str(rcode_table, r, "??"),
 			action2str(p->actions[r]));
 #endif
 
@@ -294,28 +287,34 @@ int modcall(int component, modcallable *c, REQUEST *request)
 	}
 
 	if(c == NULL) {
-		DEBUG2("modcall[%s]: NULL object returns %s",
-				comp2str[component], rcode2str[myresult]);
+		DEBUG2("modcall[%s]: NULL object returns %s for request %d",
+		       comp2str[component],
+		       lrad_int2str(rcode_table, myresult, "??"),
+		       request->number);
 		return myresult;
 	}
 
 	if(c->type==MOD_GROUP) {
 		modgroup *g = mod_callabletogroup(c);
 
-		DEBUG2("modcall: entering group %s",
-				c->name);
+		DEBUG2("modcall: entering group %s for request %d",
+				c->name, request->number);
 
 		myresult = call_modgroup(component, g, request, myresult);
 
-		DEBUG2("modcall: group %s returns %s",
-				c->name, rcode2str[myresult]);
+		DEBUG2("modcall: group %s returns %s for request %d",
+		       c->name,
+		       lrad_int2str(rcode_table, myresult, "??"),
+			   request->number);
 	} else {
 		modsingle *sp = mod_callabletosingle(c);
 
 		myresult = call_modsingle(component, sp, request, myresult);
 
-		DEBUG2("  modcall[%s]: module \"%s\" returns %s",
-			 comp2str[component], c->name, rcode2str[myresult]);
+		DEBUG2("  modcall[%s]: module \"%s\" returns %s for request %d",
+		       comp2str[component], c->name,
+		       lrad_int2str(rcode_table, myresult, "??"),
+			   request->number);
 	}
 
 	return myresult;
@@ -343,7 +342,8 @@ static void dump_mc(modcallable *c, int indent)
 
 	for(i = 0; i<RLM_MODULE_NUMCODES; ++i) {
 		DEBUG("%.*s%s = %s", indent+1, "\t\t\t\t\t\t\t\t\t\t\t",
-			rcode2str[i], action2str(c->actions[i]));
+		      lrad_int2str(rcode_table, i, "??"),
+		      action2str(c->actions[i]));
 	}
 
 	DEBUG("%.*s}", indent, "\t\t\t\t\t\t\t\t\t\t\t");
