@@ -1,7 +1,7 @@
 /*
  * util.c	Various utility functions.
  *
- * Version:     $Id: util.c,v 1.29 2002/02/20 21:35:36 aland Exp $
+ * Version:     $Id: util.c,v 1.30 2002/06/03 16:29:04 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -20,7 +20,7 @@
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: util.c,v 1.29 2002/02/20 21:35:36 aland Exp $";
+static const char rcsid[] = "$Id: util.c,v 1.30 2002/06/03 16:29:04 aland Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -39,6 +39,7 @@ static const char rcsid[] = "$Id: util.c,v 1.29 2002/02/20 21:35:36 aland Exp $"
 #endif
 
 #include "radiusd.h"
+#include "rad_assert.h"
 
 /*
  *	The signal() function in Solaris 2.5.1 sets SA_NODEFER in
@@ -86,11 +87,18 @@ void (*reset_signal(int signo, void (*func)(int)))(int)
 void request_free(REQUEST **request_ptr)
 {
 	REQUEST *request;
-	
-	if (request_ptr == NULL) 
+
+	if (request_ptr == NULL)
 		return;
+
 	request = *request_ptr;
 
+	/*
+	 *	If there's a thread currently active on this request,
+	 *	blow up!
+	 */
+	rad_assert(request->child_pid == NO_SUCH_CHILD_PID);
+	
 	if (request->packet) 
 		rad_free(&request->packet);
 
