@@ -2,7 +2,7 @@
  * valuepair.c	Valuepair functions that are radiusd-specific
  *		and as such do not belong in the library.
  *
- * Version:	$Id: valuepair.c,v 1.45 2002/10/29 21:05:22 aland Exp $
+ * Version:	$Id: valuepair.c,v 1.49.2.1 2003/09/28 13:35:49 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -22,7 +22,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: valuepair.c,v 1.45 2002/10/29 21:05:22 aland Exp $";
+static const char rcsid[] = "$Id: valuepair.c,v 1.49.2.1 2003/09/28 13:35:49 phampson Exp $";
 
 #include "autoconf.h"
 #include "libradius.h"
@@ -46,6 +46,10 @@ static const char rcsid[] = "$Id: valuepair.c,v 1.45 2002/10/29 21:05:22 aland E
 #ifndef REG_EXTENDED
 #define REG_EXTENDED (0)
 #endif
+
+#ifndef REG_NOSUB
+#define REG_NOSUB (0)
+#endif
 #endif
 
 #include "radiusd.h"
@@ -64,7 +68,7 @@ static struct cmp *cmp;
  *	Compare 2 attributes. May call the attribute compare function.
  */
 static int paircompare(REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check,
-	VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
+		       VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
 {
 	int ret = -2;
 	struct cmp *c;
@@ -235,8 +239,8 @@ int paircmp(REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check, VALUE_PAIR **r
 			 *	These are "server" check items.
 			 */
 			case PW_CRYPT_PASSWORD:
-			case PW_AUTHTYPE:
-			case PW_AUTZTYPE:
+			case PW_AUTH_TYPE:
+			case PW_AUTZ_TYPE:
 				continue;
 				break;
 
@@ -333,7 +337,7 @@ int paircmp(REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check, VALUE_PAIR **r
 
 #ifdef HAVE_REGEX_H
 			case T_OP_REG_EQ:
-				regcomp(&reg, (char *)check_item->strvalue, REG_EXTENDED);
+				regcomp(&reg, (char *)check_item->strvalue, REG_EXTENDED|REG_NOSUB);
 				compare = regexec(&reg, (char *)auth_item->strvalue,
 						0, NULL, 0);
 				regfree(&reg);
@@ -341,7 +345,7 @@ int paircmp(REQUEST *req, VALUE_PAIR *request, VALUE_PAIR *check, VALUE_PAIR **r
 				break;
 
 			case T_OP_REG_NE:
-				regcomp(&reg, (char *)check_item->strvalue, REG_EXTENDED);
+				regcomp(&reg, (char *)check_item->strvalue, REG_EXTENDED|REG_NOSUB);
 				compare = regexec(&reg, (char *)auth_item->strvalue,
 						0, NULL, 0);
 				regfree(&reg);
@@ -593,18 +597,48 @@ static int expirecmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PA
 	return +1;
 }
 
+/*
+ *	Compare the request packet type.
+ */
+static int packetcmp(void *instance, REQUEST *req, VALUE_PAIR *request,
+		     VALUE_PAIR *check,
+		     VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
+{
+	if (req->packet->code == check->lvalue) {
+		return 0;
+	}
+
+	return 1;
+}
+
+/*
+ *	Compare the response packet type.
+ */
+static int responsecmp(void *instance, REQUEST *req, VALUE_PAIR *request,
+		     VALUE_PAIR *check,
+		     VALUE_PAIR *check_pairs, VALUE_PAIR **reply_pairs)
+{
+	if (req->reply->code == check->lvalue) {
+		return 0;
+	}
+
+	return 1;
+}
+
 /*
  *	Register server-builtin special attributes.
  */
 void pair_builtincompare_init(void)
 {
-	paircompare_register(PW_NAS_PORT_ID, -1, portcmp, NULL);
+	paircompare_register(PW_NAS_PORT, -1, portcmp, NULL);
 	paircompare_register(PW_PREFIX, PW_USER_NAME, presufcmp, NULL);
 	paircompare_register(PW_SUFFIX, PW_USER_NAME, presufcmp, NULL);
 	paircompare_register(PW_CONNECT_RATE, PW_CONNECT_INFO, connectcmp, NULL);
 	paircompare_register(PW_CURRENT_TIME, 0, timecmp, NULL);
 	paircompare_register(PW_NO_SUCH_ATTRIBUTE, 0, attrcmp, NULL);
 	paircompare_register(PW_EXPIRATION, 0, expirecmp, NULL);
+	paircompare_register(PW_PACKET_TYPE, 0, packetcmp, NULL);
+	paircompare_register(PW_RESPONSE_PACKET_TYPE, 0, responsecmp, NULL);
 }
 
 /*
@@ -710,6 +744,7 @@ void pairxlatmove(REQUEST *req, VALUE_PAIR **to, VALUE_PAIR **from)
 				vp = found->next;
 				memcpy(found, i, sizeof(*found));
 				found->next = vp;
+				continue;
 			}
 			break;
 			
