@@ -2,7 +2,7 @@
  * xlat.c	Translate strings.  This is the first version of xlat 
  * 		incorporated to RADIUS
  *
- * Version:	$Id: xlat.c,v 1.54 2002/11/12 17:29:03 aland Exp $
+ * Version:	$Id: xlat.c,v 1.55 2003/03/18 05:50:54 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  */
 
 static const char rcsid[] = 
-"$Id: xlat.c,v 1.54 2002/11/12 17:29:03 aland Exp $";
+"$Id: xlat.c,v 1.55 2003/03/18 05:50:54 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -475,7 +475,7 @@ int radius_xlat(char *out, int outlen, const char *fmt,
 				p++;
 				break;
 			case 'p': /* Port number */
-				q += valuepair2str(q,freespace,pairfind(request->packet->vps,PW_NAS_PORT_ID),PW_TYPE_INTEGER, func);
+				q += valuepair2str(q,freespace,pairfind(request->packet->vps,PW_NAS_PORT),PW_TYPE_INTEGER, func);
 				p++;
 				break;
 			case 's': /* Speed */
