@@ -1,18 +1,19 @@
 /*
  * rlm_acct_unique.c
  *
- *  This program is is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License, version 2 if the
- *  License as published by the Free Software Foundation.
- * 
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *  
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Copyright 2000  The FreeRADIUS server project
  */
@@ -32,7 +33,7 @@
  */
 #define  BUFFERLEN  4096
 
-static const char rcsid[] = "$Id: rlm_acct_unique.c,v 1.27 2002/12/03 16:27:37 aland Exp $";
+static const char rcsid[] = "$Id: rlm_acct_unique.c,v 1.27.2.1 2003/09/12 19:18:03 phampson Exp $";
 
 typedef struct rlm_acct_unique_list_t {
 	DICT_ATTR		      *dattr;
