@@ -1,7 +1,7 @@
 /*
  * rlm_counter.c
  *
- * Version:  $Id: rlm_counter.c,v 1.32.2.2 2003/08/26 12:34:48 phampson Exp $
+ * Version:  $Id: rlm_counter.c,v 1.32.2.5 2003/10/04 00:20:54 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@
 
 #define UNIQUEID_MAX_LEN 32
 
-static const char rcsid[] = "$Id: rlm_counter.c,v 1.32.2.2 2003/08/26 12:34:48 phampson Exp $";
+static const char rcsid[] = "$Id: rlm_counter.c,v 1.32.2.5 2003/10/04 00:20:54 phampson Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -82,9 +82,22 @@ typedef struct rlm_counter_t {
 	time_t last_reset;	/* The time of the last reset. */
 	int dict_attr;		/* attribute number for the counter. */
 	GDBM_FILE gdbm;		/* The gdbm file handle */
+#ifdef HAVE_PTHREAD_H
 	pthread_mutex_t mutex;	/* A mutex to lock the gdbm file for only one reader/writer */
+#endif
 } rlm_counter_t;
 
+#ifndef HAVE_PTHREAD_H
+/*
+ *	This is a lot simpler than putting ifdef's around
+ *	every use of the pthread functions.
+ */
+#define pthread_mutex_lock(a)
+#define pthread_mutex_unlock(a)
+#define pthread_mutex_init(a,b)
+#define pthread_mutex_destroy(a)
+#endif
+
 typedef struct rad_counter {
 	unsigned int user_counter;
 	char uniqueid[UNIQUEID_MAX_LEN];
@@ -111,6 +124,8 @@ static CONF_PARSER module_config[] = {
   { NULL, -1, 0, NULL, NULL }
 };
 
+static int counter_detach(void *instance);
+
 
 /*
  *	See if the counter matches.
@@ -149,6 +164,7 @@ static int counter_cmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_
 	return counter.user_counter - check->lvalue;
 }
 
+
 static int add_defaults(rlm_counter_t *data)
 {
 	datum key_datum;
@@ -312,6 +328,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	data = rad_malloc(sizeof(*data));
 	if (!data) {
+		radlog(L_ERR, "rlm_counter: rad_malloc() failed.");
 		return -1;
 	}
 	memset(data, 0, sizeof(*data));
@@ -331,12 +348,14 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	if (data->key_name == NULL) {
 		radlog(L_ERR, "rlm_counter: 'key' must be set.");
-		exit(0);
+		counter_detach(data);
+		return -1;
 	}
 	dattr = dict_attrbyname(data->key_name);
 	if (dattr == NULL) {
 		radlog(L_ERR, "rlm_counter: No such attribute %s",
 				data->key_name);
+		counter_detach(data);
 		return -1;
 	}
 	data->key_attr = dattr->attr;
@@ -346,12 +365,14 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	if (data->count_attribute == NULL) {
 		radlog(L_ERR, "rlm_counter: 'count-attribute' must be set.");
-		exit(0);
+		counter_detach(data);
+		return -1;
 	}
 	dattr = dict_attrbyname(data->count_attribute);
 	if (dattr == NULL) {
 		radlog(L_ERR, "rlm_counter: No such attribute %s",
 				data->count_attribute);
+		counter_detach(data);
 		return -1;
 	}
 	data->count_attr = dattr->attr;
@@ -361,7 +382,8 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	if (data->counter_name == NULL) {
 		radlog(L_ERR, "rlm_counter: 'counter-name' must be set.");
-		exit(0);
+		counter_detach(data);
+		return -1;
 	}
 
 	memset(&flags, 0, sizeof(flags));
@@ -370,6 +392,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	if (dattr == NULL) {
 		radlog(L_ERR, "rlm_counter: Failed to create counter attribute %s",
 				data->counter_name);
+		counter_detach(data);
 		return -1;
 	}
 	data->dict_attr = dattr->attr;
@@ -381,13 +404,15 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	if (data->check_name == NULL) {
 		radlog(L_ERR, "rlm_counter: 'check-name' must be set.");
-		exit(0);
+		counter_detach(data);
+		return -1;
 	}
 	dict_addattr(data->check_name, 0, PW_TYPE_INTEGER, -1, flags);
 	dattr = dict_attrbyname(data->check_name);
 	if (dattr == NULL) {
 		radlog(L_ERR, "rlm_counter: Failed to create check attribute %s",
 				data->counter_name);
+		counter_detach(data);
 		return -1;
 	}
 	data->check_attr = dattr->attr;
@@ -399,6 +424,7 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 		if ((dval = dict_valbyname(PW_SERVICE_TYPE, data->service_type)) == NULL) {
 			radlog(L_ERR, "rlm_counter: Failed to find attribute number for %s",
 					data->service_type);
+			counter_detach(data);
 			return -1;
 		}
 		data->service_val = dval->value;
@@ -409,24 +435,30 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	if (data->reset == NULL) {
 		radlog(L_ERR, "rlm_counter: 'reset' must be set.");
-		exit(0);
+		counter_detach(data);
+		return -1;
 	}
 	now = time(NULL);
 	data->reset_time = 0;
 	data->last_reset = now;
 
-	if (find_next_reset(data,now) == -1)
+	if (find_next_reset(data,now) == -1){
+		radlog(L_ERR, "rlm_counter: find_next_reset() returned -1. Exiting.");
+		counter_detach(data);
 		return -1;
+	}
 
 	if (data->filename == NULL) {
 		radlog(L_ERR, "rlm_counter: 'filename' must be set.");
-		exit(0);
+		counter_detach(data);
+		return -1;
 	}
 	data->gdbm = gdbm_open(data->filename, sizeof(int),
 			GDBM_WRCREAT | GDBM_COUNTER_OPTS, 0600, NULL);
 	if (data->gdbm == NULL) {
 		radlog(L_ERR, "rlm_counter: Failed to open file %s: %s",
 				data->filename, strerror(errno));
+		counter_detach(data);
 		return -1;
 	}
 	if (gdbm_setopt(data->gdbm, GDBM_CACHESIZE, &cache_size, sizeof(int)) == -1)
@@ -458,8 +490,11 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 
 			data->last_reset = now;
 			ret = reset_db(data);
-			if (ret != RLM_MODULE_OK)
+			if (ret != RLM_MODULE_OK){
+				radlog(L_ERR, "rlm_counter: reset_db() failed");
+				counter_detach(data);
 				return -1;
+			}
 		}
 		else
 			data->reset_time = next_reset;
@@ -474,8 +509,11 @@ static int counter_instantiate(CONF_SECTION *conf, void **instance)
 	}
 	else{
 		ret = add_defaults(data);
-		if (ret != RLM_MODULE_OK)
+		if (ret != RLM_MODULE_OK){
+			radlog(L_ERR, "rlm_counter: add_defaults() failed");
+			counter_detach(data);
 			return -1;
+		}
 	}
 
 
@@ -549,7 +587,19 @@ static int counter_accounting(void *instance, REQUEST *request)
 			DEBUG("rlm_counter: This Service-Type is not allowed. Returning NOOP.");
 			return RLM_MODULE_NOOP;
 		}
-	}	
+	}
+	/*
+	 * Check if request->timestamp - {Acct-Delay-Time} < last_reset
+	 * If yes reject the packet since it is very old
+	 */
+	key_vp = pairfind(request->packet->vps, PW_ACCT_DELAY_TIME);
+	if (key_vp != NULL){
+		if (key_vp->lvalue != 0 && (request->timestamp - key_vp->lvalue) < data->last_reset){
+			DEBUG("rlm_counter: This packet is too old. Returning NOOP.");
+			return RLM_MODULE_NOOP;
+		}
+	}
+
 	
 
 	/*
@@ -804,12 +854,15 @@ static int counter_detach(void *instance)
 	rlm_counter_t *data = (rlm_counter_t *) instance;
 
 	paircompare_unregister(data->dict_attr, counter_cmp);
-	gdbm_close(data->gdbm);
+	if (data->gdbm)
+		gdbm_close(data->gdbm);
 	free(data->filename);
 	free(data->reset);
 	free(data->key_name);
 	free(data->count_attribute);
 	free(data->counter_name);
+	free(data->check_name);
+	free(data->service_type);
 	pthread_mutex_destroy(&data->mutex);
 
 	free(instance);
