@@ -1,21 +1,25 @@
 /*
  * rlm_dbm.c authorize:    authorize using ndbm database
  *
- * Version:     $Id: rlm_dbm.c,v 1.5 2002/10/15 15:06:11 aland Exp $
+ * Version:     $Id: rlm_dbm.c,v 1.6.2.1 2003/09/12 19:18:03 phampson Exp $
  *
- *  This program is is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License, version 2 if the
- *  License as published by the Free Software Foundation.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- *  Copyright 2001 Koulik Andrei, Sandy Service
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * Copyright 2001 Koulik Andrei, Sandy Service
  */
 
-
 #include <string.h>
 #include <stdlib.h>
 
@@ -41,7 +45,7 @@
 #	include "sandymod.h"
 #endif
 
-static const char rcsid[] = "$Id: rlm_dbm.c,v 1.5 2002/10/15 15:06:11 aland Exp $";
+static const char rcsid[] = "$Id: rlm_dbm.c,v 1.6.2.1 2003/09/12 19:18:03 phampson Exp $";
 
 #define MYDBM	DBM
 #define get_user_content dbm_fetch
@@ -284,7 +288,11 @@ static int rlm_dbm_instantiate(CONF_SECTION *conf, void **instance) {
 	struct rlm_dbm_t *inst;
 	
 	inst = rad_malloc(sizeof(rlm_dbm_t));
-	
+	if (!inst) {
+		return -1;
+	}
+	memset(inst, 0, sizeof(*inst));
+
         if (cf_section_parse(conf, inst, module_config) < 0) {
                 free(inst);      
                 return -1;
