@@ -1,23 +1,28 @@
 /*
  * rlm_dbm_parser.c :    Create dbm file from plain text
  *
- * Version:     $Id: rlm_dbm_parser.c,v 1.6 2002/10/15 14:51:18 aland Exp $
+ * Version:     $Id: rlm_dbm_parser.c,v 1.7.2.1 2003/09/12 19:18:04 phampson Exp $
  *
- *  This program is is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License, version 2 if the
- *  License as published by the Free Software Foundation.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
  *
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
  *
- *  Copyright 2001 Koulik Andrei, Sandy Service
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * Copyright 2001 Koulik Andrei, Sandy Service
  */
 
 
 char sccsid[] =
-"$Id: rlm_dbm_parser.c,v 1.6 2002/10/15 14:51:18 aland Exp $ sandy module project\n Copyright 2001 Sandy Service\nCopyright 2001 Koulik Andrei";
+"$Id: rlm_dbm_parser.c,v 1.7.2.1 2003/09/12 19:18:04 phampson Exp $ sandy module project\n Copyright 2001 Sandy Service\nCopyright 2001 Koulik Andrei";
 
 #include "autoconf.h"
 #include <fcntl.h>
@@ -290,7 +295,7 @@ static int sm_parse_file(FILE*fp,const char* fname) {
 
 
 static void sm_usage(void) {
-	fprintf(stderr, "Usage: %s [-c] [-d raddb] [-f imputfile] [-o outputfile] [-x] [-v] [-q] [username1 [username2] ...]\n\n",progname);
+	fprintf(stderr, "Usage: %s [-c] [-d raddb] [-i inputfile] [-o outputfile] [-x] [-v] [-q] [username1 [username2] ...]\n\n",progname);
 
 	fprintf(stderr, "-c	create new database.\n");
 	fprintf(stderr, "-x	debug mode.\n");
@@ -330,7 +335,7 @@ int main(int n,char **argv) {
 				print_stat = 0;
 				break;
 			case 'v':
-				printf("%s: $Id: rlm_dbm_parser.c,v 1.6 2002/10/15 14:51:18 aland Exp $ \n",progname);
+				printf("%s: $Id: rlm_dbm_parser.c,v 1.7.2.1 2003/09/12 19:18:04 phampson Exp $ \n",progname);
 				exit(0);
 			case 'c':
 				oflags = O_CREAT | O_TRUNC | O_RDWR;
