@@ -1,25 +1,26 @@
 /*
  * rlm_detail.c	accounting:    Write the "detail" files.
  *
- * Version:	$Id: rlm_detail.c,v 1.28 2002/08/24 16:54:57 aland Exp $
+ * Version:	$Id: rlm_detail.c,v 1.30.2.1 2003/09/12 19:18:04 phampson Exp $
  *
- *  This program is is free software; you can redistribute it and/or modify
- *  it under the terms of the GNU General Public License, version 2 if the
- *  License as published by the Free Software Foundation.
- * 
- *  This program is distributed in the hope that it will be useful,
- *  but WITHOUT ANY WARRANTY; without even the implied warranty of
- *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *  GNU General Public License for more details.
- *  
- *  You should have received a copy of the GNU General Public License
- *  along with this program; if not, write to the Free Software
- *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  * Copyright 2000  The FreeRADIUS server project
  */
 
-static const char rcsid[] = "$Id: rlm_detail.c,v 1.28 2002/08/24 16:54:57 aland Exp $";
+static const char rcsid[] = "$Id: rlm_detail.c,v 1.30.2.1 2003/09/12 19:18:04 phampson Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -72,7 +73,11 @@ static int detail_instantiate(CONF_SECTION *conf, void **instance)
 {
 	struct detail_instance *inst;
 
-	inst = rad_malloc(sizeof *inst);
+	inst = rad_malloc(sizeof(*inst));
+	if (!inst) {
+		return -1;
+	}
+	memset(inst, 0, sizeof(*inst));
 
 	if (cf_section_parse(conf, inst, module_config) < 0) {
 		free(inst);
@@ -85,16 +90,12 @@ static int detail_instantiate(CONF_SECTION *conf, void **instance)
 	return 0;
 }
 
-/*
- *	Accounting - write the detail files.
- */
-static int detail_accounting(void *instance, REQUEST *request)
+static int do_detail(void *instance, REQUEST *request, VALUE_PAIR *pair)
 {
 	int		outfd;
 	FILE		*outfp;
 	char		buffer[DIRLEN];
 	char		*p;
-	VALUE_PAIR	*pair;
 	int		ret = RLM_MODULE_OK;
 	struct stat	st;
 	int		locked;
@@ -224,7 +225,6 @@ static int detail_accounting(void *instance, REQUEST *request)
 		fputs(ctime_r(&request->timestamp, buffer), outfp);
 
 		/* Write each attribute/value to the log file */
-		pair = request->packet->vps;
 		while (pair) {
 			if (pair->attribute != PW_PASSWORD) {
 				fputs("\t", outfp);
@@ -271,6 +271,31 @@ static int detail_accounting(void *instance, REQUEST *request)
 	return ret;
 }
 
+/*
+ *	Accounting - write the detail files.
+ */
+static int detail_accounting(void *instance, REQUEST *request)
+{
+
+	return do_detail(instance,request,request->packet->vps);
+}
+
+/*
+ *	Incoming Access Request - write the detail files.
+ */
+static int detail_authorize(void *instance, REQUEST *request)
+{
+	return do_detail(instance,request,request->packet->vps);
+}
+
+/*
+ *	Outgoing Access-Request Reply - write the detail files.
+ */
+static int detail_postauth(void *instance, REQUEST *request)
+{
+	return do_detail(instance,request,request->reply->vps);
+}
+
 
 /*
  *	Clean up.
@@ -295,13 +320,13 @@ module_t rlm_detail = {
 	detail_instantiate,		/* instantiation */
 	{
 		NULL,			/* authentication */
-		NULL,		 	/* authorization */
+		detail_authorize, 	/* authorization */
 		NULL,			/* preaccounting */
 		detail_accounting,	/* accounting */
 		NULL,			/* checksimul */
 		NULL,			/* pre-proxy */
 		NULL,			/* post-proxy */
-		NULL			/* post-auth */
+		detail_postauth		/* post-auth */
 	},
 	detail_detach,			/* detach */
 	NULL				/* destroy */
