@@ -1,7 +1,7 @@
 /*
  * rlm_chap.c
  *
- * Version:  $Id: rlm_digest.c,v 1.2 2002/07/17 17:37:28 aland Exp $
+ * Version:  $Id: rlm_digest.c,v 1.3 2002/08/24 16:54:57 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -33,7 +33,7 @@
 #include "conffile.h"
 #include "rad_assert.h"
 
-static const char rcsid[] = "$Id: rlm_digest.c,v 1.2 2002/07/17 17:37:28 aland Exp $";
+static const char rcsid[] = "$Id: rlm_digest.c,v 1.3 2002/08/24 16:54:57 aland Exp $";
 
 static int digest_authorize(void *instance, REQUEST *request)
 {
@@ -497,7 +497,10 @@ module_t rlm_digest = {
 		digest_authorize, 	/* authorization */
 		NULL,			/* preaccounting */
 		NULL,			/* accounting */
-		NULL			/* checksimul */
+		NULL,			/* checksimul */
+		NULL,			/* pre-proxy */
+		NULL,			/* post-proxy */
+		NULL			/* post-auth */
 	},
 	NULL,				/* detach */
 	NULL,				/* destroy */
