@@ -1,7 +1,7 @@
 /*
  * eap.c    rfc2284 & rfc2869 implementation
  *
- * Version:     $Id: eap.c,v 1.10 2002/09/18 21:07:42 cparker Exp $
+ * Version:     $Id: eap.c,v 1.22.2.1 2003/10/17 16:42:06 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -59,31 +59,43 @@ static const char *eap_types[] = {
   "",   
   "identity",
   "notification",
-  "nak",
+  "nak",			/* NAK */
   "md5",
   "otp",
   "gtc",
-  "",
-  "",
-  "",
-  "",
-  "",
-  "",
-  "tls"
+  "7",
+  "8",
+  "9",
+  "10",
+  "11",
+  "12",
+  "tls",			/* 13 */
+  "14",
+  "15",
+  "16",
+  "leap",			/* 17 */
+  "18",
+  "19",
+  "20",
+  "ttls",			/* 21 */
+  "22",
+  "23",
+  "24",
+  "peap"			/* 25 */
 };
 
 /*
  * Load all the required eap authentication types.
  * Get all the supported EAP-types from config file. 
  */
-int eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs)
+int eaptype_load(EAP_TYPES **type_list, const char *type_name,
+		 CONF_SECTION *cs)
 {
 	EAP_TYPES **last, *node;
 	lt_dlhandle handle;
 	char auth_type_name[NAME_LEN];
 	int i;
 
-	memset(auth_type_name, 0, NAME_LEN);
 	snprintf(auth_type_name, sizeof(auth_type_name), "rlm_eap_%s", type_name);
 
 	last = type_list;
@@ -116,7 +128,7 @@ int eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs)
 	node->typeid = 0;
 	memset(node->typename, 0, NAME_LEN);
 	node->type_stuff = NULL;
-	snprintf(node->typename, sizeof(type_name), "%s", type_name);
+	strNcpy(node->typename, type_name, sizeof(node->typename));
 	for (i = PW_EAP_MAX_TYPES; i > 0; i--) {
 		if (!strcmp(type_name, eap_types[i])) {
 			node->typeid = i;
@@ -147,7 +159,7 @@ int eaptype_load(EAP_TYPES **type_list, const char *type_name, CONF_SECTION *cs)
 		return -1;
 	}
 
-	radlog(L_INFO, "rlm_eap: Loaded and initialized the type %s", type_name);
+	DEBUG("rlm_eap: Loaded and initialized the type %s", type_name);
 	*last = node;
 	return 0;
 }
@@ -186,12 +198,12 @@ int eaptype_call(int eap_type, operation_t action,
 
 	atype = eaptype_byid(&type_list, eap_type);
 	if (!atype) {
-		radlog(L_INFO, "rlm_eap: Unsupported EAP_TYPE %d",
+		radlog(L_ERR, "rlm_eap: Unsupported EAP_TYPE %d",
 			handler->eap_ds->response->type.type);
 		return 0;
 	}
 
-	radlog(L_INFO, "rlm_eap: processing type %s", atype->typename);
+	DEBUG2("  rlm_eap: processing type %s", atype->typename);
 
 	switch (action) {
 	case INITIATE:
@@ -237,11 +249,13 @@ int eaptype_select(EAP_TYPES *type_list, EAP_HANDLER *handler, char *conftype)
 	eaptype = &handler->eap_ds->response->type;
 	switch(eaptype->type) {
 	case PW_EAP_IDENTITY:
+		DEBUG2("  rlm_eap: EAP Identity");
 		if (eaptype_call(type, INITIATE, type_list, handler) == 0)
 			return EAP_INVALID;
 			break;
 
 	case PW_EAP_NAK:
+		DEBUG2("  rlm_eap: EAP NAK");
 		/*
 		 * It is invalid to request identity, notification & nak in nak
 		 */
@@ -249,9 +263,15 @@ int eaptype_select(EAP_TYPES *type_list, EAP_HANDLER *handler, char *conftype)
 			(eaptype->data[0] < PW_EAP_MD5)) {
 			return EAP_INVALID;
 		}
+
+		/*
+		 *	The one byte of NAK data is the preferred EAP type
+		 *	of the client.
+		 */
 		switch (eaptype->data[0]) {
 		case PW_EAP_MD5:
 		case PW_EAP_TLS:
+		case PW_EAP_LEAP:
 			/*
 			 * eap-type specified in typdata is supported
 			 */
@@ -260,6 +280,8 @@ int eaptype_select(EAP_TYPES *type_list, EAP_HANDLER *handler, char *conftype)
 				return EAP_INVALID;
 			break;
 		default :
+			DEBUG2("  rlm_eap: Unknown EAP type %d, reverting to default_eap_type",
+			       eaptype->data[0]);
 			/*
 			 * Unsupported type, default to configured one.
 			 * or rather reject it
@@ -274,8 +296,9 @@ int eaptype_select(EAP_TYPES *type_list, EAP_HANDLER *handler, char *conftype)
 		case PW_EAP_OTP:
 		case PW_EAP_GTC:
 		case PW_EAP_TLS:
+		case PW_EAP_LEAP:
 		default:
-			radlog(L_INFO, "rlm_eap: EAP_TYPE - %s",
+			DEBUG2("  rlm_eap: EAP_TYPE - %s",
 				eap_types[eaptype->type]);
 			if (eaptype_call(eaptype->type, AUTHENTICATE,
 				type_list, handler) == 0) {
@@ -295,62 +318,85 @@ int eaptype_select(EAP_TYPES *type_list, EAP_HANDLER *handler, char *conftype)
  */
 eap_packet_t *eap_attribute(VALUE_PAIR *vps)
 {
-	VALUE_PAIR *vp_list, *vp;
+	VALUE_PAIR *first, *vp;
 	eap_packet_t *eap_packet;
 	unsigned char *ptr;
 	uint16_t len;
 	int total_len;
 
-	/* Get only EAP-Message attribute list */
-	vp_list = paircopy2(vps, PW_EAP_MESSAGE);
-	if (vp_list == NULL) {
-		radlog(L_ERR, "rlm_eap: EAP_Message not found");
+	/*
+	 *	Get only EAP-Message attribute list
+	 */
+	first = pairfind(vps, PW_EAP_MESSAGE);
+	if (first == NULL) {
+		radlog(L_ERR, "rlm_eap: EAP-Message not found");
 		return NULL;
 	}
 
 	/*
-	 * Get the Actual length from the EAP packet
-	 * First EAP-Message contains the EAP packet header
+	 *	Sanity check the length before doing anything.
 	 */
-	memcpy(&len, vp_list->strvalue+2, sizeof(uint16_t));
+	if (first->length < 4) {
+		radlog(L_ERR, "rlm_eap: EAP packet is too short.");
+		return NULL;
+	}
+
+	/*
+	 *	Get the Actual length from the EAP packet
+	 *	First EAP-Message contains the EAP packet header
+	 */
+	memcpy(&len, first->strvalue + 2, sizeof(len));
 	len = ntohs(len);
 
-	eap_packet = (eap_packet_t *)malloc(len);
-	if (eap_packet == NULL) {
-		radlog(L_ERR, "rlm_eap: out of memory");
-		pairfree(&vp_list);
+	/*
+	 *	Take out even more weird things.
+	 */
+	if (len < 4) {
+		radlog(L_ERR, "rlm_eap: EAP packet has invalid length.");
 		return NULL;
 	}
-	ptr = (unsigned char *)eap_packet;
-	memcpy(ptr, vp_list->strvalue, vp_list->length);
-	ptr += vp_list->length;
-
-	total_len = vp_list->length;
 
 	/*
-	 * TODO: This check can also be based on Framed-MTU
-	if (len < MAX_STRING_LEN)
-		return eap_packet;
+	 *	Sanity check the length, BEFORE malloc'ing memory.
 	 */
-
-	if (vp_list->next == NULL) {
-		pairfree(&vp_list);
-		return eap_packet;
+	total_len = 0;
+	for (vp = first; vp; vp = pairfind(vp->next, PW_EAP_MESSAGE)) {
+		total_len += vp->length;
+		
+		if (total_len > len) {
+			radlog(L_ERR, "rlm_eap: Malformed EAP packet.  Length in packet header does not match actual length");
+			return NULL;
+		}
 	}
 
-	radlog(L_INFO, "rlm_eap: Multiple EAP_Message attributes found");
+	/*
+	 *	If the length is SMALLER, die, too.
+	 */
+	if (total_len < len) {
+		radlog(L_ERR, "rlm_eap: Malformed EAP packet.  Length in packet header does not match actual length");
+		return NULL;
+	}
+
+	/*
+	 *	Now that we know the lengths are OK, allocate memory.
+	 */
+	eap_packet = (eap_packet_t *) malloc(len);
+	if (eap_packet == NULL) {
+		radlog(L_ERR, "rlm_eap: out of memory");
+		return NULL;
+	}
+
+	/*
+	 *	Copy the data from EAP-Message's over to out EAP packet.
+	 */
+	ptr = (unsigned char *)eap_packet;
 
 	/* RADIUS ensures order of attrs, so just concatenate all */
-	for (vp = vp_list->next; vp; vp = vp->next) {
-		if (total_len + vp->length > len) {
-			radlog(L_ERR, "rlm_eap: Malformed EAP, lengths mismatch");
-		}
-		total_len += vp->length;
+	for (vp = first; vp; vp = pairfind(vp->next, PW_EAP_MESSAGE)) {
 		memcpy(ptr, vp->strvalue, vp->length);
 		ptr += vp->length;
 	}
 
-	pairfree(&vp_list);
 	return eap_packet;
 }
 
@@ -410,7 +456,7 @@ int eap_wireformat(EAP_PACKET *reply)
  * Set the RADIUS reply codes based on EAP request codes
  * Append any additonal VPs to RADIUS reply
  */
-int eap_compose(REQUEST *request, EAP_PACKET *reply)
+int eap_compose(REQUEST *request, EAP_DS *eap_ds)
 {
 	uint16_t eap_len, len;
 	VALUE_PAIR *eap_msg;
@@ -418,6 +464,7 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
 	VALUE_PAIR *vp;
 	eap_packet_t *eap_packet;
 	unsigned char 	*ptr;
+	EAP_PACKET *reply = eap_ds->request;
 
 	/*
 	 * ID serves to suppport request/response
@@ -431,15 +478,25 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
 	 */
 	eap_msg = pairfind(request->packet->vps, PW_EAP_MESSAGE);
 	rq = (EAP_PACKET *)eap_msg->strvalue;
-	reply->id = rq->id;
 
-	switch (reply->code) {
+	/*
+	 *	The ID for the EAP packet to the NAS wasn't set.
+	 *	Do so now.
+	 */
+	if (!eap_ds->set_request_id) {
+		reply->id = rq->id;
+		
+		switch (reply->code) {
 		case PW_EAP_SUCCESS:
 		case PW_EAP_FAILURE:
 	    		break;
-
+			
 		default:
 	    		++reply->id;
+		}
+	} else {
+		DEBUG2("  rlm_eap: Underlying EAP-Type set EAP ID to %d",
+		       reply->id);
 	}
 
 	if (eap_wireformat(reply) == EAP_INVALID) {
@@ -452,15 +509,9 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
 	ptr = (unsigned char *)eap_packet;
 
 	do {
-		/* TODO: Fix it in radius.c
-
-		if (eap_len > MAX_STRING_LEN) {
-			len = MAX_STRING_LEN;
-			eap_len -= MAX_STRING_LEN;
-		*/
-		if (eap_len > 252) {
-			len = 252;
-			eap_len -= 252;
+		if (eap_len > 253) {
+			len = 253;
+			eap_len -= 253;
 		} else {
 			len = eap_len;
 			eap_len = 0;
@@ -470,7 +521,7 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
 		 * create a value pair & append it to the request reply list
 		 * This memory gets freed up when request is freed up
 		 */
-		eap_msg = paircreate(PW_EAP_MESSAGE, PW_TYPE_STRING);
+		eap_msg = paircreate(PW_EAP_MESSAGE, PW_TYPE_OCTETS);
 		memcpy(eap_msg->strvalue, ptr, len);
 		eap_msg->length = len;
 		pairadd(&(request->reply->vps), eap_msg);
@@ -479,25 +530,32 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
 	} while (eap_len);
 
 	/*
-	 * EAP-Message is always associated with Message-Authenticator
-	 * but not viceversa.
+	 *	EAP-Message is always associated with
+	 *	Message-Authenticator but not vice-versa.
+	 *
+	 *	Don't add a Message-Authenticator if it's already
+	 *	there.
 	 */
-	vp = paircreate(PW_MESSAGE_AUTHENTICATOR, PW_TYPE_OCTETS);
-	memset(vp->strvalue, 0, AUTH_VECTOR_LEN);
-	vp->length = AUTH_VECTOR_LEN;
-	pairadd(&(request->reply->vps), vp);
+	vp = pairfind(request->reply->vps, PW_MESSAGE_AUTHENTICATOR);
+	if (!vp) {
+		vp = paircreate(PW_MESSAGE_AUTHENTICATOR, PW_TYPE_OCTETS);
+		memset(vp->strvalue, 0, AUTH_VECTOR_LEN);
+		vp->length = AUTH_VECTOR_LEN;
+		pairadd(&(request->reply->vps), vp);
+	}
 
 	/*
 	 * Generate State, only if it not Identity request
 	 */ 
 	if ((eap_packet->code == PW_EAP_REQUEST) &&
-		(eap_packet->data[0] >= PW_EAP_MD5)) {
+	    (eap_packet->data[0] >= PW_EAP_MD5)) {
 		vp = generate_state();
 		pairadd(&(request->reply->vps), vp);
 	}
 		
-	/* Set request reply code */
-	switch(reply->code) {
+	/* Set request reply code, but only if it's not already set. */
+	if (!request->reply->code) switch(reply->code) {
+	case PW_EAP_RESPONSE:
 	case PW_EAP_SUCCESS:
 		request->reply->code = PW_AUTHENTICATION_ACK;
 		break;
@@ -507,10 +565,9 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
 	case PW_EAP_REQUEST:
 		request->reply->code = PW_ACCESS_CHALLENGE;
 		break;
-	case PW_EAP_RESPONSE:
 	default:
 		/* Should never enter here */
-		radlog(L_ERR, "rlm_eap: reply code is unknown, Reject it.");
+		radlog(L_ERR, "rlm_eap: reply code %d is unknown, Rejecting the request.", reply->code);
 		request->reply->code = PW_AUTHENTICATION_REJECT;
 		break;
 	}
@@ -523,27 +580,72 @@ int eap_compose(REQUEST *request, EAP_PACKET *reply)
  */
 int eap_start(REQUEST *request)
 {
+	VALUE_PAIR *vp;
 	VALUE_PAIR *eap_msg;
 	EAP_DS *eapstart;
 
 	eap_msg = pairfind(request->packet->vps, PW_EAP_MESSAGE);
 	if (eap_msg == NULL) {
-		radlog(L_ERR, "rlm_eap: EAP-Message not found");
 		return EAP_NOOP;
 	}
 
+	/*
+	 *  http://www.freeradius.org/rfc/rfc2869.html#EAP-Message
+	 */
+	vp = pairfind(request->packet->vps, PW_MESSAGE_AUTHENTICATOR);
+	if (!vp) {
+		radlog(L_ERR, "rlm_eap: EAP-Message without Message-Authenticator: Ignoring the request due to RFC 2869 Section 5.13 requirements");
+		return EAP_NOOP;
+	}
+
+	if ((eap_msg->strvalue[0] == 0) ||
+	    (eap_msg->strvalue[0] > PW_EAP_MAX_TYPES)) {
+		DEBUG2("  rlm_eap: Unknown EAP packet");
+	} else {
+		DEBUG2("  rlm_eap: EAP packet type %s id %d length %d",
+		       eap_types[eap_msg->strvalue[0]],
+		       eap_msg->strvalue[1],
+		       (eap_msg->strvalue[2] << 8) | eap_msg->strvalue[3]);
+	}
+
+	/*
+	 *	If we've been configured to proxy, do nothing.
+	 *
+	 *	Note that we don't check if the realm is local.
+	 *	We figure that anyone bright enough to add
+	 *	Proxy-To-Realm is bright enough to NOT do so
+	 *	when it's a local realm.
+	 */
+	if (pairfind(request->config_items, PW_PROXY_TO_REALM) != NULL) {
+	  	return EAP_NOOP;
+	}
+
+	/*
+	 *	Not a start message.  Don't start anything.
+	 *
+	 *	Later EAP messages are longer than the 'start' message,
+	 *	so this function returns 'no start found', so that
+	 *	the rest of the EAP code can use the State attribute
+	 *	to match this EAP-Message to an ongoing conversation.
+	 */
 	if (eap_msg->length != EAP_START) {
+		DEBUG2("  rlm_eap: EAP Start not found");
 		return EAP_NOTFOUND;
 	}
 
-	radlog(L_INFO, "rlm_eap: Got EAP_START message");
+	DEBUG2("  rlm_eap: Got EAP_START message");
 	if ((eapstart = eap_ds_alloc()) == NULL) {
+		DEBUG2("  rlm_eap: EAP Start failed in allocation");
 		return EAP_FAIL;
 	}
+
+	/*
+	 *	Hmm... why isn't this taken from the eap_msg?
+	 */
 	eapstart->request->code = PW_EAP_REQUEST;
 	eapstart->request->type.type = PW_EAP_IDENTITY;
 
-	eap_compose(request, eapstart->request);
+	eap_compose(request, eapstart);
 
 	eap_ds_free(&eapstart);
 	return EAP_FOUND;
@@ -552,19 +654,19 @@ int eap_start(REQUEST *request)
 /*
  * compose EAP FAILURE packet in EAP-Message
  */
-void eap_fail(REQUEST *request, EAP_PACKET *reply)
+void eap_fail(REQUEST *request, EAP_DS *eap_ds)
 {
-	reply->code = PW_EAP_FAILURE;
-	eap_compose(request, reply);
+	eap_ds->request->code = PW_EAP_FAILURE;
+	eap_compose(request, eap_ds);
 }
 
 /*
  * compose EAP SUCCESS packet in EAP-Message
  */
-void eap_success(REQUEST *request, EAP_PACKET *reply)
+void eap_success(REQUEST *request, EAP_DS *eap_ds)
 {
-	reply->code = PW_EAP_SUCCESS;
-	eap_compose(request, reply);
+	eap_ds->request->code = PW_EAP_SUCCESS;
+	eap_compose(request, eap_ds);
 }
 
 /*
@@ -579,9 +681,10 @@ int eap_validation(eap_packet_t *eap_packet)
 
 	/* High level EAP packet checks */
 	if ((len <= EAP_HEADER_LEN) ||
-		(eap_packet->code != PW_EAP_RESPONSE) ||
-		(eap_packet->data[0] <= 0) ||
-		(eap_packet->data[0] > PW_EAP_MAX_TYPES)) {
+ 	    ((eap_packet->code != PW_EAP_RESPONSE) &&
+ 	     (eap_packet->code != PW_EAP_REQUEST)) ||
+	    (eap_packet->data[0] <= 0) ||
+	    (eap_packet->data[0] > PW_EAP_MAX_TYPES)) {
 
 		radlog(L_AUTH, "rlm_eap: Incorrect EAP Message, "
 				"Ignoring the packet");
@@ -658,6 +761,58 @@ char *eap_identity(eap_packet_t *eap_packet)
 	return identity;
 }
 
+
+/*
+ * Create our Request-Response data structure with the eap packet
+ */
+static EAP_DS *eap_buildds(eap_packet_t **eap_packet_p)
+{
+	EAP_DS *eap_ds = NULL;
+	eap_packet_t	*eap_packet = NULL;
+	int typelen;
+	uint16_t len;
+
+	eap_packet = *eap_packet_p;
+	if (eap_packet == NULL) {
+		return NULL;
+	}
+
+	if ((eap_ds = eap_ds_alloc()) == NULL) {
+		return NULL;
+	}
+	eap_ds->response->packet = (unsigned char *)eap_packet;
+        eap_ds->response->code = eap_packet->code;
+        eap_ds->response->id = eap_packet->id;
+        eap_ds->response->type.type = eap_packet->data[0];
+
+	memcpy(&len, eap_packet->length, sizeof(uint16_t));
+	len = ntohs(len);
+	eap_ds->response->length = len;
+
+	/* First byte in eap_packet->data is *EAP-Type* */
+	/*
+	 * First 5 bytes, in eap, are code+id+length(2)+type
+	 * The rest is TypeData
+	 * skip *type* while getting typedata from data
+	 */
+	typelen = len - 5/*code+id+length+type*/;
+	if (typelen > 0) {
+		/*
+		 * Since packet contians the complete eap_packet, 
+		 * typedata will be a ptr in packet to its typedata
+		 */
+		eap_ds->response->type.data = eap_ds->response->packet + 5/*code+id+length+type*/;
+		eap_ds->response->type.length = typelen;
+	} else {
+		eap_ds->response->type.length = 0;
+		eap_ds->response->type.data = NULL;
+	}
+
+	*eap_packet_p = NULL;
+	return eap_ds;
+}
+
+
 /*
  * If identity response then create a fresh handler & fill the identity
  * else handler MUST be in our list, get that.
@@ -681,7 +836,6 @@ EAP_HANDLER *eap_handler(EAP_HANDLER **list, eap_packet_t **eap_packet_p, REQUES
 	 * EAP_HANDLER MUST be found in the list if it is not EAP-Identity response
 	 */
 	if (eap_packet->data[0] != PW_EAP_IDENTITY) {
-
 		unique = eap_regenerateid(request, eap_packet->id);
 		if (unique == NULL) {
 			return NULL;
@@ -697,7 +851,6 @@ EAP_HANDLER *eap_handler(EAP_HANDLER **list, eap_packet_t **eap_packet_p, REQUES
 			return NULL;
 		}
 	} else {
-
 		handler = eap_handler_alloc();
 		if (handler == NULL) {
 			radlog(L_ERR, "rlm_eap: out of memory");
@@ -747,7 +900,7 @@ EAP_HANDLER *eap_handler(EAP_HANDLER **list, eap_packet_t **eap_packet_p, REQUES
 		 */
 		handler->configured = paircopy(request->config_items);
 		if (handler->configured == NULL) {
-			radlog(L_INFO, "rlm_eap: No configured information for this user");
+			DEBUG2("  rlm_eap: No configured information for this user");
 
 			/*
 			 * FIXME: If there is no config info then
@@ -769,59 +922,10 @@ EAP_HANDLER *eap_handler(EAP_HANDLER **list, eap_packet_t **eap_packet_p, REQUES
 
 	handler->timestamp = time(NULL);
 	handler->reply_vps = &(request->reply->vps);
+	handler->request = request; /* LEAP needs this */
 	return handler;
 }
 
-/*
- * Create our Request-Response data structure with the eap packet
- */
-EAP_DS *eap_buildds(eap_packet_t **eap_packet_p)
-{
-	EAP_DS *eap_ds = NULL;
-	eap_packet_t	*eap_packet = NULL;
-	int typelen;
-	uint16_t len;
-
-	eap_packet = *eap_packet_p;
-	if (eap_packet == NULL) {
-		return NULL;
-	}
-
-	if ((eap_ds = eap_ds_alloc()) == NULL) {
-		return NULL;
-	}
-	eap_ds->response->packet = (unsigned char *)eap_packet;
-        eap_ds->response->code = eap_packet->code;
-        eap_ds->response->id = eap_packet->id;
-        eap_ds->response->type.type = eap_packet->data[0];
-
-	memcpy(&len, eap_packet->length, sizeof(uint16_t));
-	len = ntohs(len);
-	eap_ds->response->length = len;
-
-	/* First byte in eap_packet->data is *EAP-Type* */
-	/*
-	 * First 5 bytes, in eap, are code+id+length(2)+type
-	 * The rest is TypeData
-	 * skip *type* while getting typedata from data
-	 */
-	typelen = len - 5/*code+id+length+type*/;
-	if (typelen > 0) {
-		/*
-		 * Since packet contians the complete eap_packet, 
-		 * typedata will be a ptr in packet to its typedata
-		 */
-		eap_ds->response->type.data = eap_ds->response->packet + 5/*code+id+length+type*/;
-		eap_ds->response->type.length = typelen;
-	} else {
-		eap_ds->response->type.length = 0;
-		eap_ds->response->type.data = NULL;
-	}
-
-	*eap_packet_p = NULL;
-	return eap_ds;
-}
-
 
 /*
  * Regenerate the ID to match the ID stored in the list.
@@ -829,25 +933,12 @@ EAP_DS *eap_buildds(eap_packet_t **eap_packet_p)
  */
 unsigned char *eap_regenerateid(REQUEST *request, unsigned char response_id)
 {
-	VALUE_PAIR 	*nas = NULL;
 	VALUE_PAIR 	*state = NULL;
 	unsigned char	*id = NULL;
 
-	/* This check should be in the server code */
-	nas = pairfind(request->packet->vps, PW_NAS_IP_ADDRESS);
-	if (nas == NULL) {
-		nas = pairfind(request->packet->vps, PW_NAS_IDENTIFIER);
-		if (nas == NULL) {
-			radlog(L_ERR, "rlm_eap: Invalid RADIUS packet." 
-				" Both NAS-IP-Address & NAS-Identifier "
-				"are missing");
-			return NULL;
-		}
-	}
-
 	state = pairfind(request->packet->vps, PW_STATE);
 	if (state == NULL) {
-		radlog(L_INFO, "rlm_eap: NO State Attribute found.");
+		DEBUG2("  rlm_eap: NO State Attribute found: Cannot match EAP packet to any existing conversation.");
 		return NULL;
 	}
 	if (verify_state(state) != 0) {
@@ -855,7 +946,7 @@ unsigned char *eap_regenerateid(REQUEST *request, unsigned char response_id)
 		return NULL;
 	}
 
-	id = (unsigned char *)malloc(1/*Length*/ + 1/*Id*/ + state->length + nas->length);
+	id = (unsigned char *)malloc(1/*Length*/ + 1/*Id*/ + state->length + sizeof(request->packet->src_ipaddr));
 	if (id == NULL) {
 		radlog(L_ERR, "rlm_eap: out of memory");
 		return NULL;
@@ -863,12 +954,16 @@ unsigned char *eap_regenerateid(REQUEST *request, unsigned char response_id)
 
 	/*
 	 * Generate unique-id to check for the reply 
-	 * id = Length + ID + State + (NAS-IP-Address | NAS-Identifier)
+	 * id = Length + ID + State + Client IP Address
+	 *
+	 *  Note that we do NOT use NAS-IP-Address, or NAS-Identifier,
+	 *  as they may lie to us!
 	 */
-	id[0] = (1 + 1 + state->length + nas->length) & 0xFF;
+	id[0] = (1 + 1 + state->length + sizeof(request->packet->src_ipaddr)) & 0xFF;
 	memcpy(id+1, &response_id, sizeof(unsigned char));
 	memcpy(id+2, state->strvalue, state->length);
-	memcpy(id+2+state->length, nas->strvalue, nas->length);
+	memcpy(id+2+state->length, &request->packet->src_ipaddr,
+	       sizeof(request->packet->src_ipaddr));
 
 	return id;
 }
@@ -879,29 +974,16 @@ unsigned char *eap_regenerateid(REQUEST *request, unsigned char response_id)
  */
 unsigned char *eap_generateid(REQUEST *request, unsigned char response_id)
 {
-	VALUE_PAIR 	*nas = NULL;
 	VALUE_PAIR 	*state = NULL;
 	unsigned char	*id = NULL;
 
-	/* This check should be in the server code */
-	nas = pairfind(request->packet->vps, PW_NAS_IP_ADDRESS);
-	if (nas == NULL) {
-		nas = pairfind(request->packet->vps, PW_NAS_IDENTIFIER);
-		if (nas == NULL) {
-			radlog(L_ERR, "rlm_eap: Invalid RADIUS packet." 
-				" Both NAS-IP-Address & NAS-Identifier "
-				"are missing");
-			return NULL;
-		}
-	}
-
 	state = pairfind(request->reply->vps, PW_STATE);
 	if (state == NULL) {
-		radlog(L_INFO, "rlm_eap: NO State Attribute found.");
+		DEBUG2("  rlm_eap: NO State Attribute found.  Cannot match the EAP packet to any existing conversation.");
 		return NULL;
 	}
 
-	id = (unsigned char *)malloc(1/*Length*/ + 1/*Id*/ + state->length + nas->length);
+	id = (unsigned char *)malloc(1/*Length*/ + 1/*Id*/ + state->length + sizeof(request->packet->src_ipaddr));
 	if (id == NULL) {
 		radlog(L_ERR, "rlm_eap: out of memory");
 		return NULL;
@@ -909,12 +991,16 @@ unsigned char *eap_generateid(REQUEST *request, unsigned char response_id)
 
 	/*
 	 * Generate unique-id to check for the reply 
-	 * id = Length + ID + State + (NAS-IP-Address | NAS-Identifier)
+	 * id = Length + ID + State + Client IP Address
+	 *
+	 *  Note that we do NOT use NAS-IP-Address, or NAS-Identifier,
+	 *  as they may lie to us!
 	 */
-	id[0] = (1 + 1 + state->length + nas->length) & 0xFF;
+	id[0] = (1 + 1 + state->length + sizeof(request->packet->src_ipaddr)) & 0xFF;
 	memcpy(id+1, &response_id, sizeof(unsigned char));
 	memcpy(id+2, state->strvalue, state->length);
-	memcpy(id+2+state->length, nas->strvalue, nas->length);
+	memcpy(id+2+state->length, &request->packet->src_ipaddr,
+               sizeof(request->packet->src_ipaddr));
 
 	return id;
 }
