@@ -1,7 +1,7 @@
 /*
  * eap_tls.c
  *
- * Version:     $Id: eap_tls.c,v 1.11 2002/09/17 13:33:50 cparker Exp $
+ * Version:     $Id: eap_tls.c,v 1.13 2003/04/04 20:06:12 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -391,6 +391,22 @@ eaptls_status_t eaptls_verify(EAP_DS *eap_ds, EAP_DS *prev_eap_ds)
  * Points to consider during EAP-TLS data extraction
  * 1. In the received packet, No data will be present incase of ACK-NAK
  * 2. Incase if more fragments need to be received then ACK after retreiving this fragment.
+ *
+ *  RFC 2716 Section 4.2.  PPP EAP TLS Request Packet
+ *
+ *  0                   1                   2                   3
+ *  0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1 2 3 4 5 6 7 8 9 0 1
+ *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+ *  |     Code      |   Identifier  |            Length             |
+ *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+ *  |     Type      |     Flags     |      TLS Message Length
+ *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+ *  |     TLS Message Length        |       TLS Data...
+ *  +-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
+ *
+ *  The Length field is two octets and indicates the length of the EAP
+ *  packet including the Code, Identifir, Length, Type, and TLS data
+ *  fields.
  */
 EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 {
@@ -402,6 +418,16 @@ EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 	if (status  == EAPTLS_INVALID)
 		return NULL;
 
+	/*
+	 *	We have a 'type' after the EAP header, but no more data.
+	 *
+	 *	For TLS, we always have 'flags'
+	 */
+	if (eap_ds->response->length <= 2) {
+		radlog(L_ERR, "rlm_eap_tls: Invalid EAP-TLS packet received.  (No data.)");
+		return NULL;
+	}
+
 	tlspacket = eaptls_alloc();
 	if (tlspacket == NULL) return NULL;
 
@@ -409,14 +435,25 @@ EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 	 * Code & id for EAPTLS & EAP are same
 	 * but eaptls_length = eap_length - 1(EAP-Type = 1 octet)
 	 *
-	 * length = code + id + length + flags + tlsdata
+	 * length = code + id + length + type + tlsdata
 	 *        =  1   +  1 +   2    +  1    +  X
 	 */
 	tlspacket->code = eap_ds->response->code;
 	tlspacket->id = eap_ds->response->id;
-	tlspacket->length = eap_ds->response->length - 1/*EAPtype*/;
+	tlspacket->length = eap_ds->response->length - 1; /* EAP type */
 	tlspacket->flags = eap_ds->response->type.data[0];
 
+	/*
+	 *	A quick sanity check of the flags.  If we've been told
+	 *	that there's a length, and there isn't one, then stop.
+	 */
+	if ((tlspacket->flags & 0x80) &&
+	    (tlspacket->length < 5)) { /* flags + TLS message length */
+		radlog(L_ERR, "rlm_eap_tls: Invalid EAP-TLS packet received.  (Length bit is set, but no length was found.)");
+		eaptls_free(&tlspacket);
+		return NULL;
+	}
+
 	switch (status) {
 	/*
 	   The TLS Message Length field is
@@ -437,6 +474,12 @@ EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 	case EAPTLS_FIRST_FRAGMENT:
 	case EAPTLS_LENGTH_INCLUDED:
 	case EAPTLS_MORE_FRAGMENTS_WITH_LENGTH:
+		if (tlspacket->length < 5) { /* flags + TLS message length */
+			radlog(L_ERR, "rlm_eap_tls: Invalid EAP-TLS packet received.  (Excepted length, got none.)");
+			eaptls_free(&tlspacket);
+			return NULL;
+		}
+
 		/*
 		 * Extract all the TLS fragments from the previous eap_ds
 		 * Start appending this fragment to the above ds
@@ -445,12 +488,19 @@ EAPTLS_PACKET *eaptls_extract(EAP_DS *eap_ds, eaptls_status_t status)
 		data_len = ntohl(data_len);
 		data = (eap_ds->response->type.data + 5/*flags+TLS-Length*/);
 		len = eap_ds->response->type.length - 5/*flags+TLS-Length*/;
-		if(data_len > len) {
+
+		/*
+		 *	Hmm... this should be an error, too.
+		 */
+		if (data_len > len) {
 			radlog(L_INFO, "Total Length Included");
 			data_len = len;
 		}
 		break;
 
+		/*
+		 *	Data length is implicit, from the EAP header.
+		 */
 	case EAPTLS_MORE_FRAGMENTS:
 	case EAPTLS_OK:
 		data_len = eap_ds->response->type.length - 1/*flags*/;
@@ -498,7 +548,7 @@ int eaptls_compose(EAP_DS *eap_ds, EAPTLS_PACKET *reply)
    Identifier value in the subsequent fragment contained within an EAP-
    Reponse.
 */
-	eap_ds->request->type.data = malloc(reply->length - TLS_HEADER_LEN);
+	eap_ds->request->type.data = malloc(reply->length - TLS_HEADER_LEN + 1);
 	if (eap_ds->request->type.data == NULL) {
 		radlog(L_ERR, "rlm_eap_tls: out of memory");
 		return 0;
@@ -510,8 +560,7 @@ int eaptls_compose(EAP_DS *eap_ds, EAPTLS_PACKET *reply)
 	ptr = eap_ds->request->type.data;
 	*ptr++ = (uint8_t)(reply->flags & 0xFF);
 
-	//memcpy(ptr, reply->data, reply->length - 4/*EAPTLS_Header*/ - 1/*flags*/);
-	memcpy(ptr, reply->data, reply->dlen);
+	if (reply->dlen) memcpy(ptr, reply->data, reply->dlen);
 
 	switch (reply->code) {
 	case EAPTLS_ACK:
