@@ -1,7 +1,7 @@
 /*
  * rlm_expr.c
  *
- * Version:	$Id: rlm_expr.c,v 1.4 2002/10/01 15:27:02 aland Exp $
+ * Version:	$Id: rlm_expr.c,v 1.5.2.1 2003/07/14 17:31:48 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -32,7 +32,7 @@
 #include "modules.h"
 #include "conffile.h"
 
-static const char rcsid[] = "$Id: rlm_expr.c,v 1.4 2002/10/01 15:27:02 aland Exp $";
+static const char rcsid[] = "$Id: rlm_expr.c,v 1.5.2.1 2003/07/14 17:31:48 aland Exp $";
 
 /*
  *	Define a structure for our module configuration.
@@ -74,7 +74,7 @@ static expr_map_t map[] =
 static int get_number(REQUEST *request, const char **string, int *answer)
 {
 	int		i, found;
-	int		result, x;
+	uint32_t       	result, x;
 	const char	*p;
 	expr_token_t	this;
 
@@ -221,6 +221,8 @@ static int expr_xlat(void *instance, REQUEST *request, char *fmt, char *out, int
 	const		char *p;
 	char		buffer[256];
 
+	inst = inst;		/* -Wunused */
+
 	/*
 	 * Do an xlat on the provided string (nice recursive operation).
 	 */
@@ -267,6 +269,8 @@ static int expr_instantiate(CONF_SECTION *conf, void **instance)
 	 */
 	
 	inst = rad_malloc(sizeof(rlm_expr_t));
+	if (!inst)
+		return -1;
 	memset(inst, 0, sizeof(rlm_expr_t));
 		
 	xlat_name = cf_section_name2(conf);
