@@ -2,7 +2,7 @@
  * rlm_fastusers.c	authorization: Find a user in the hashed "users" file.
  *                	accounting:    Do nothing.  Auth module only.
  *
- * Version:	$Id: rlm_fastusers.c,v 1.26 2002/08/24 16:54:57 aland Exp $
+ * Version:	$Id: rlm_fastusers.c,v 1.28.2.1 2003/07/14 17:31:48 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -98,6 +98,27 @@ static int fallthrough(VALUE_PAIR *vp)
 	return tmp ? tmp->lvalue : 0;
 }
 
+/*
+ *       returncheck - Check for Auth-Type = Reject and return appropriate
+ *                     module return code if it is found.
+ */
+static int rad_check_return(VALUE_PAIR *list)
+{
+      VALUE_PAIR      *authtype;
+
+      /* 
+       * We check for Auth-Type = Reject here
+       */
+
+      authtype = pairfind(list, PW_AUTHTYPE);
+      if((authtype) && authtype->lvalue == PW_AUTHTYPE_REJECT)  {
+              DEBUG2("rad_check_return:  Auth-Type is Reject");
+              return RLM_MODULE_REJECT;
+      }
+
+      return RLM_MODULE_UPDATED;
+}
+
 static int fastuser_buildhash(struct fastuser_instance *inst) {
 	long memsize=0;
 	int rcode, hashindex;
@@ -523,7 +544,8 @@ static int fastuser_instantiate(CONF_SECTION *conf, void **instance)
 	struct fastuser_instance *inst=0;
 
 	inst = rad_malloc(sizeof *inst);
-
+	if (!inst)
+		return -1;
 	memset(inst, 0, sizeof(*inst));
 
 	if (cf_section_parse(conf, inst, module_config) < 0) {
@@ -648,15 +670,15 @@ static int fastuser_authorize(void *instance, REQUEST *request)
 			pairmove(&request->reply->vps, &reply_tmp);
 			pairfree(&reply_tmp);
 
-		}
+			/*                         
+			 * There's no fallthru on this default which    
+			 * is *before* we find the user in the file,         
+			 * so we know it's safe to quit here                        
+			 */
+			if (!fallthrough(curdefault->reply))
+			  break;
 
-		/* 
-		 * There's no fallthru on this default which
-		 * is *before* we find the user in the file, 
-		 * so we know it's safe to quit here
-		 */
-		if (!fallthrough(curdefault->reply))
-			break;
+		}
 
 		/*
 		 * If we found the user, we want to stop
