@@ -1,3 +1,23 @@
+/*
+ * rlm_passwd.c
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * Copyright 2000  The FreeRADIUS server project
+ */
+
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
@@ -347,6 +367,7 @@ static CONF_PARSER module_config[] = {
 	   offsetof(struct passwd_instance, allowmultiple), NULL,  "no" },
 	{ "hashsize",   PW_TYPE_INTEGER,
 	   offsetof(struct passwd_instance, hashsize), NULL,  "100" },
+	{ NULL, -1, 0, NULL, NULL }
 };
 
 static int passwd_instantiate(CONF_SECTION *conf, void **instance)
@@ -464,7 +485,7 @@ static int passwd_detach (void *instance) {
 #undef inst
 }
 
-static void addresult (struct passwd_instance * inst, VALUE_PAIR ** vp, struct mypasswd * pw, char when, char *listname)
+static void addresult (struct passwd_instance * inst, VALUE_PAIR ** vp, struct mypasswd * pw, char when, const char *listname)
 {
 	int i;
 	VALUE_PAIR *newpair;
