@@ -3,7 +3,7 @@
  *		Contains the functions for the "huntgroups" and "hints"
  *		files.
  *
- * Version:     $Id: rlm_preprocess.c,v 1.41 2002/08/24 16:54:59 aland Exp $
+ * Version:     $Id: rlm_preprocess.c,v 1.45 2003/07/07 19:17:31 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Alan DeKok <aland@ox.org>
  */
 
-static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.41 2002/08/24 16:54:59 aland Exp $";
+static const char rcsid[] = "$Id: rlm_preprocess.c,v 1.45 2003/07/07 19:17:31 aland Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -110,7 +110,7 @@ static void ascend_nasport_hack(VALUE_PAIR *nas_port, int channels_per_line)
  */
 static void cisco_vsa_hack(VALUE_PAIR *vp)
 {
-	int		vendorpec, vendorcode;
+	int		vendorcode;
 	char		*ptr;
 	char		newattr[MAX_STRING_LEN];
 
@@ -118,18 +118,53 @@ static void cisco_vsa_hack(VALUE_PAIR *vp)
 		vendorcode = (vp->attribute >> 16); /* HACK! */
 		if (vendorcode == 0) continue;	/* ignore non-VSA attributes */
 
-		vendorpec  = dict_vendorpec(vendorcode);
-		if (vendorpec == 0) continue; /* ignore unknown VSA's */
+		if (vendorcode == 0) continue; /* ignore unknown VSA's */
 
-		if (vendorpec != 9) continue; /* not a Cisco VSA, continue */
+		if (vendorcode != 9) continue; /* not a Cisco VSA, continue */
 
-		if ((vp->attribute & 0xffff) == 1) continue; /* Cisco-AVPair */
+		if (vp->type != PW_TYPE_STRING) continue;
 
 		/*
-		 *  We strip out the duplicity from the value field,
-		 *  we use only the value on the right side of = character.
+		 *  No weird packing.  Ignore it.
 		 */
-		if ((ptr = strchr(vp->strvalue, '=')) != NULL) {
+		ptr = strchr(vp->strvalue, '='); /* find an '=' */
+		if (!ptr) continue;
+
+		/*
+		 *	Cisco-AVPair's get packed as:
+		 *
+		 *	Cisco-AVPair = "h323-foo-bar = baz"
+		 *
+		 *	which makes sense only if you're a lunatic.
+		 *	This code looks for the attribute named inside
+		 *	of the string, and if it exists, adds it as a new
+		 *	attribute.
+		 */
+		if ((vp->attribute & 0xffff) == 1) {
+			char *p;
+			DICT_ATTR	*dattr;
+
+			p = vp->strvalue;
+			getword(&p, newattr, sizeof(newattr));
+
+			if (((dattr = dict_attrbyname(newattr)) != NULL) &&
+			    (dattr->type == PW_TYPE_STRING)) {
+				VALUE_PAIR *newvp;
+				
+				/*
+				 *  Make a new attribute.
+				 */
+				newvp = pairmake(newattr, ptr + 1, T_OP_EQ);
+				if (newvp) {
+					pairadd(&vp, newvp);
+				}
+			}
+		} else {	/* h322-foo-bar = "h323-foo-bar = baz" */
+			/*
+			 *	We strip out the duplicity from the
+			 *	value field, we use only the value on
+			 *	the right side of the '=' character.
+			 */
 			strNcpy(newattr, ptr + 1, sizeof(newattr));
 			strNcpy((char *)vp->strvalue, newattr,
 				sizeof(vp->strvalue));
@@ -542,6 +577,7 @@ static void add_nas_attr(REQUEST *request)
 			exit(1);
 		}
 		nas->lvalue = request->packet->src_ipaddr;
+		ip_hostname(nas->strvalue, sizeof(nas->strvalue), nas->lvalue);
 		pairadd(&request->packet->vps, nas);
 	}
 
@@ -579,6 +615,7 @@ static int preprocess_instantiate(CONF_SECTION *conf, void **instance)
 	 *	Allocate room to put the module's instantiation data.
 	 */
 	data = (rlm_preprocess_t *) rad_malloc(sizeof(*data));
+	memset(data, 0, sizeof(*data));
 
 	/*
 	 *	Read this modules configuration data.
@@ -645,7 +682,7 @@ static int preprocess_authorize(void *instance, REQUEST *request)
 		 *	approaching rationality.
 		 */
 		ascend_nasport_hack(pairfind(request->packet->vps,
-					     PW_NAS_PORT_ID),
+					     PW_NAS_PORT),
 				    data->ascend_channels_per_line);
 	}
 
