@@ -1,7 +1,7 @@
 /*
  * rlm_realm.c	
  *
- * Version:	$Id: rlm_realm.c,v 1.43 2002/08/24 16:55:00 aland Exp $
+ * Version:	$Id: rlm_realm.c,v 1.46 2003/07/07 19:10:39 aland Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -39,7 +39,7 @@
 #include "radiusd.h"
 #include "modules.h"
 
-static const char rcsid[] = "$Id: rlm_realm.c,v 1.43 2002/08/24 16:55:00 aland Exp $";
+static const char rcsid[] = "$Id: rlm_realm.c,v 1.46 2003/07/07 19:10:39 aland Exp $";
 
 #define  REALM_FORMAT_PREFIX   0
 #define  REALM_FORMAT_SUFFIX   1
@@ -65,7 +65,6 @@ static CONF_PARSER module_config[] = {
  */
 static REALM *check_for_realm(void *instance, REQUEST *request)
 {
-	int is_local;
 	char namebuf[MAX_STRING_LEN];
 	char *username;
 	char *realmname = NULL;
@@ -87,7 +86,7 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 	 */
 	if ((request->proxy != NULL) ||
 	    (request->username == NULL)) {
-		DEBUG2("    rlm_realm: Proxy reply, or no user name.  Ignoring.");
+		DEBUG2("    rlm_realm: Proxy reply, or no User-Name.  Ignoring.");
 		return NULL;
 	}
 
@@ -145,7 +144,7 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 	 *	what's going on.
 	 */
 	if (realmname) {
-		DEBUG2("    rlm_realm: Looking up realm %s for User-Name = \"%s\"",
+		DEBUG2("    rlm_realm: Looking up realm \"%s\" for User-Name = \"%s\"",
 		       realmname, request->username->strvalue);
 	} else {
 		DEBUG2("    rlm_realm: No '%c' in User-Name = \"%s\", looking up realm NULL",
@@ -157,11 +156,11 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 	 */
 	realm = realm_find(realmname, (request->packet->code == PW_ACCOUNTING_REQUEST));
 	if (!realm) {
-		DEBUG2("    rlm_realm: No such realm %s",
+		DEBUG2("    rlm_realm: No such realm \"%s\"",
 		       (realmname == NULL) ? "NULL" : realmname);
 		return NULL;
 	}
-	DEBUG2("    rlm_realm: Found realm %s", realm->realm);
+	DEBUG2("    rlm_realm: Found realm \"%s\"", realm->realm);
 	
 	/*
 	 *	If we've been told to strip the realm off, then do so.
@@ -190,7 +189,7 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 		request->username = vp;
 	}
 
-	DEBUG2("  rlm_realm: Proxying request from user %s to realm %s",
+	DEBUG2("    rlm_realm: Proxying request from user %s to realm %s",
 	       username, realm->realm);
 
 	/*
@@ -203,10 +202,9 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 	/*
 	 *	Figure out what to do with the request.
 	 */
-	is_local = FALSE;
 	switch (request->packet->code) {
 	default:
-		DEBUG2("rlm_realm: Unknown packet code %d\n",
+		DEBUG2("    rlm_realm: Unknown packet code %d\n",
 		       request->packet->code);
 		return NULL;		/* don't do anything */
 		
@@ -215,12 +213,12 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 		 */
 	case PW_ACCOUNTING_REQUEST:
 		if (realm->acct_ipaddr == htonl(INADDR_NONE)) {
-			DEBUG2("rlm_realm:  Accounting realm is LOCAL.");
-			is_local = TRUE;
+			DEBUG2("    rlm_realm: Accounting realm is LOCAL.");
+			return NULL;
 		}
 
 		if (realm->acct_port == 0) {
-			DEBUG2("rlm_realm:  acct_port is not set.  proxy cancelled");
+			DEBUG2("    rlm_realm: acct_port is not set.  Proxy cancelled.");
 			return NULL;
 		}
 		break;
@@ -230,24 +228,17 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 		 */
 	case PW_AUTHENTICATION_REQUEST:
 		if (realm->ipaddr == htonl(INADDR_NONE)) {
-			DEBUG2("rlm_realm:  Authentication realm is LOCAL.");
-			is_local = TRUE;
+			DEBUG2("    rlm_realm: Authentication realm is LOCAL.");
+			return NULL;
 		}
 
 		if (realm->auth_port == 0) {
-			DEBUG2("rlm_realm:  auth_port is not set.  proxy cancelled");
+			DEBUG2("    rlm_realm: auth_port is not set.  Proxy cancelled.");
 			return NULL;
 		}
 		break;
 	}
 
-	/*
-	 *	Local realm, don't proxy it.
-	 */
-	if (is_local) {
-		return NULL;
-	}
-
 	/*
 	 *      If this request has arrived from another freeradius server
 	 *      that has already proxied the request, we don't need to do
@@ -257,12 +248,12 @@ static REALM *check_for_realm(void *instance, REQUEST *request)
 		if (vp->attribute == PW_FREERADIUS_PROXIED_TO) {
 			if (request->packet->code == PW_AUTHENTICATION_REQUEST &&
 			    vp->lvalue == realm->ipaddr) {
-				DEBUG2("rlm_realm: Request not proxied due to Freeradius-Proxied-To");
+				DEBUG2("    rlm_realm: Request not proxied due to Freeradius-Proxied-To");
 				return NULL;
 			}
 			if (request->packet->code == PW_ACCOUNTING_REQUEST &&
 			    vp->lvalue == realm->acct_ipaddr) {
-				DEBUG2("rlm_realm: Request not proxied due to Freeradius-Proxied-To");
+				DEBUG2("    rlm_realm: Request not proxied due to Freeradius-Proxied-To");
 				return NULL;
 			}
 		}
@@ -304,7 +295,11 @@ static int realm_instantiate(CONF_SECTION *conf, void **instance)
         struct realm_config_t *inst;
 
         /* setup a storage area for instance data */
-        inst = rad_malloc(sizeof(struct realm_config_t));
+        inst = rad_malloc(sizeof(*inst));
+	if (!inst) {
+		return -1;
+	}
+	memset(inst, 0, sizeof(*inst));
 
 	if(cf_section_parse(conf, inst, module_config) < 0) {
 	       free(inst);
@@ -360,7 +355,7 @@ static int realm_authorize(void *instance, REQUEST *request)
 	/*
 	 *	Maybe add a Proxy-To-Realm attribute to the request.
 	 */
-	DEBUG2("rlm_realm:  Preparing to proxy authentication request to realm %s\n",
+	DEBUG2("    rlm_realm: Preparing to proxy authentication request to realm \"%s\"\n",
 	       realm->realm);
 	add_proxy_to_realm(&request->config_items, realm);
 
@@ -394,11 +389,11 @@ static int realm_preacct(void *instance, REQUEST *request)
 	/*
 	 *	Maybe add a Proxy-To-Realm attribute to the request.
 	 */
-	DEBUG2("rlm_realm:  Preparing to proxy accounting request to realm %s\n",
+	DEBUG2("    rlm_realm: Preparing to proxy accounting request to realm \"%s\"\n",
 	       realm->realm);
 	add_proxy_to_realm(&request->config_items, realm);
 
-	return RLM_MODULE_OK; /* try the next module */
+	return RLM_MODULE_UPDATED; /* try the next module */
 }
 
 static int realm_detach(void *instance)
