@@ -1,5 +1,5 @@
 /*
- * $Id: db_oracle.sql,v 1.3 2002/11/08 21:48:09 cparker Exp $
+ * $Id: db_oracle.sql,v 1.4.2.3 2003/07/12 13:53:12 pnixon Exp $
  *
  * Oracle schema for FreeRADIUS
  *
@@ -7,22 +7,12 @@
  * NOTE: Which columns are NULLable??
  */
 
-/*
- * Table structure for table 'dictionary'
- */
-CREATE TABLE dictionary (
-	id		INT PRIMARY KEY,
-	type		VARCHAR(30),
-	attribute	VARCHAR(32),
-	value		VARCHAR(32),
-	format		VARCHAR(20),
-	vendor		VARCHAR(32)
-);
-CREATE SEQUENCE dictionary_seq START WITH 1 INCREMENT BY 1;
-
 /*
  * Table structure for table 'nas'
+ * This is not currently used but FreeRADIUS but is usefull for reporting
+ * anyway.
  */
+
 CREATE TABLE nas (
 	id 		INT PRIMARY KEY,
 	nasname		VARCHAR(128),
@@ -32,7 +22,8 @@ CREATE TABLE nas (
 	ports		INT,
 	secret		VARCHAR(60),
 	community	VARCHAR(50),
-	snmp		VARCHAR(10)
+	snmp		VARCHAR(10),
+	naslocation	VARCHAR(10)
 );
 CREATE SEQUENCE nas_seq START WITH 1 INCREMENT BY 1;
 
@@ -56,8 +47,8 @@ CREATE TABLE radacct (
 	connectinfo_stop	VARCHAR(32),
 	acctinputoctets		NUMERIC(12),
 	acctoutputoctets	NUMERIC(12),
-	calledstationid		VARCHAR(10),
-	callingstationid	VARCHAR(10),
+	calledstationid		VARCHAR(50),
+	callingstationid	VARCHAR(50),
 	acctterminatecause	VARCHAR(32),
 	servicetype		VARCHAR(32),
 	framedprotocol		VARCHAR(32),
