@@ -1,12 +1,24 @@
-/***************************************************************************
-*  sql_sybase.c                        rlm_sql - FreeRADIUS SQL Module     *
-*                                                                          *
-*      Sybase (ctlibrary) routines for rlm_sql                             *
-*                                                                          *
-*      Error handling stolen from Sybase example code "firstapp.c"         *
-*                                                                          *
-*                                     Mattias Sjostrom <mattias@nogui.se>  *
-***************************************************************************/
+/*
+ * sql_sybase.c	Sybase (ctlibrary) routines for rlm_sql
+ *		Error handling stolen from Sybase example code "firstapp.c"
+ *
+ *   This program is free software; you can redistribute it and/or modify
+ *   it under the terms of the GNU General Public License as published by
+ *   the Free Software Foundation; either version 2 of the License, or
+ *   (at your option) any later version.
+ *
+ *   This program is distributed in the hope that it will be useful,
+ *   but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *   GNU General Public License for more details.
+ *
+ *   You should have received a copy of the GNU General Public License
+ *   along with this program; if not, write to the Free Software
+ *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ *
+ * Copyright 2000  The FreeRADIUS server project
+ * Copyright 2000  Mattias Sjostrom <mattias@nogui.se>
+ */
 
 #include <stdio.h>
 #include <sys/stat.h>
