@@ -2,7 +2,7 @@
  * rlm_sql.c		SQL Module
  * 		Main SQL module file. Most ICRADIUS code is located in sql.c
  *
- * Version:	$Id: rlm_sql.c,v 1.108.2.1 2002/12/04 16:59:29 aland Exp $
+ * Version:	$Id: rlm_sql.c,v 1.116.2.3 2003/09/28 13:21:09 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -24,7 +24,7 @@
  */
 
 static const char rcsid[] =
-	"$Id: rlm_sql.c,v 1.108.2.1 2002/12/04 16:59:29 aland Exp $";
+	"$Id: rlm_sql.c,v 1.116.2.3 2003/09/28 13:21:09 phampson Exp $";
 
 #include "autoconf.h"
 
@@ -49,42 +49,78 @@ static const char rcsid[] =
 #include "rad_assert.h"
 
 static CONF_PARSER module_config[] = {
-	{"driver",PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_driver), NULL, "mysql"},
-	{"server",PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_server), NULL, "localhost"},
-	{"port",PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_port), NULL, ""},
-	{"login", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_login), NULL, ""},
-	{"password", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_password), NULL, ""},
-	{"radius_db", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_db), NULL, "radius"},
-	{"acct_table", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_acct_table), NULL, "radacct"},
-	{"acct_table2", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_acct_table2), NULL, "radacct"},
-	{"authcheck_table", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_authcheck_table), NULL, "radcheck"},
-	{"authreply_table", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_authreply_table), NULL, "radreply"},
-	{"groupcheck_table", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_groupcheck_table), NULL, "radgroupcheck"},
-	{"groupreply_table", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_groupreply_table), NULL, "radgroupreply"},
-	{"usergroup_table", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_usergroup_table), NULL, "usergroup"},
-	{"nas_table", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_nas_table), NULL, "nas"},
-	{"dict_table", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,sql_dict_table), NULL, "dictionary"},
-	{"sqltrace", PW_TYPE_BOOLEAN, offsetof(SQL_CONFIG,sqltrace), NULL, "0"},
-	{"sqltracefile", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,tracefile), NULL, SQLTRACEFILE},
-	{"deletestalesessions", PW_TYPE_BOOLEAN, offsetof(SQL_CONFIG,deletestalesessions), NULL, "0"},
-	{"num_sql_socks", PW_TYPE_INTEGER, offsetof(SQL_CONFIG,num_sql_socks), NULL, "5"},
-	{"sql_user_name", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,query_user), NULL, ""},
-	{"default_user_profile", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,default_profile), NULL, ""},
-	{"query_on_not_found", PW_TYPE_BOOLEAN, offsetof(SQL_CONFIG,query_on_not_found), NULL, "no"},
-	{"authorize_check_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authorize_check_query), NULL, ""},
-	{"authorize_reply_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authorize_reply_query), NULL, ""},
-	{"authorize_group_check_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authorize_group_check_query), NULL, ""},
-	{"authorize_group_reply_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,authorize_group_reply_query), NULL, ""},
-	{"accounting_onoff_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_onoff_query), NULL, ""},
-	{"accounting_update_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_update_query), NULL, ""},
-	{"accounting_start_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_start_query), NULL, ""},
-	{"accounting_start_query_alt", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_start_query_alt), NULL, ""},
-	{"accounting_stop_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_stop_query), NULL, ""},
-	{"accounting_stop_query_alt", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,accounting_stop_query_alt), NULL, ""},
-	{"group_membership_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,groupmemb_query), NULL, ""},
-	{"connect_failure_retry_delay", PW_TYPE_INTEGER, offsetof(SQL_CONFIG,connect_failure_retry_delay), NULL, "60"},
-	{"simul_count_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,simul_count_query), NULL, ""},
-	{"simul_verify_query", PW_TYPE_STRING_PTR, offsetof(SQL_CONFIG,simul_verify_query), NULL, ""},
+	{"driver",PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_driver), NULL, "mysql"},
+	{"server",PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_server), NULL, "localhost"},
+	{"port",PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_port), NULL, ""},
+	{"login", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_login), NULL, ""},
+	{"password", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_password), NULL, ""},
+	{"radius_db", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_db), NULL, "radius"},
+	{"acct_table", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_acct_table), NULL, "radacct"},
+	{"acct_table2", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_acct_table2), NULL, "radacct"},
+	{"authcheck_table", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_authcheck_table), NULL, "radcheck"},
+	{"authreply_table", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_authreply_table), NULL, "radreply"},
+	{"groupcheck_table", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_groupcheck_table), NULL, "radgroupcheck"},
+	{"groupreply_table", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_groupreply_table), NULL, "radgroupreply"},
+	{"usergroup_table", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_usergroup_table), NULL, "usergroup"},
+	{"nas_table", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_nas_table), NULL, "nas"},
+	{"dict_table", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,sql_dict_table), NULL, "dictionary"},
+	{"sqltrace", PW_TYPE_BOOLEAN,
+	 offsetof(SQL_CONFIG,sqltrace), NULL, "no"},
+	{"sqltracefile", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,tracefile), NULL, SQLTRACEFILE},
+	{"deletestalesessions", PW_TYPE_BOOLEAN,
+	 offsetof(SQL_CONFIG,deletestalesessions), NULL, "no"},
+	{"num_sql_socks", PW_TYPE_INTEGER,
+	 offsetof(SQL_CONFIG,num_sql_socks), NULL, "5"},
+	{"sql_user_name", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,query_user), NULL, ""},
+	{"default_user_profile", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,default_profile), NULL, ""},
+	{"query_on_not_found", PW_TYPE_BOOLEAN,
+	 offsetof(SQL_CONFIG,query_on_not_found), NULL, "no"},
+	{"authorize_check_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,authorize_check_query), NULL, ""},
+	{"authorize_reply_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,authorize_reply_query), NULL, ""},
+	{"authorize_group_check_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,authorize_group_check_query), NULL, ""},
+	{"authorize_group_reply_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,authorize_group_reply_query), NULL, ""},
+	{"accounting_onoff_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,accounting_onoff_query), NULL, ""},
+	{"accounting_update_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,accounting_update_query), NULL, ""},
+	{"accounting_start_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,accounting_start_query), NULL, ""},
+	{"accounting_start_query_alt", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,accounting_start_query_alt), NULL, ""},
+	{"accounting_stop_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,accounting_stop_query), NULL, ""},
+	{"accounting_stop_query_alt", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,accounting_stop_query_alt), NULL, ""},
+	{"group_membership_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,groupmemb_query), NULL, ""},
+	{"connect_failure_retry_delay", PW_TYPE_INTEGER,
+	 offsetof(SQL_CONFIG,connect_failure_retry_delay), NULL, "60"},
+	{"simul_count_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,simul_count_query), NULL, ""},
+	{"simul_verify_query", PW_TYPE_STRING_PTR,
+	 offsetof(SQL_CONFIG,simul_verify_query), NULL, ""},
 
 	{NULL, -1, 0, NULL, NULL}
 };
@@ -104,21 +140,33 @@ static int rlm_sql_init(void) {
 	return 0;
 }
 
+/*
+ *	Yucky prototype.
+ */
+static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, const char *username);
+
 /*
  *	sql xlat function. Right now only SELECTs are supported. Only
  *	the first element of the SELECT result will be used.
  */
-static int sql_xlat(void *instance, REQUEST *request, char *fmt, char *out, int freespace,
-			RADIUS_ESCAPE_STRING func)
+static int sql_xlat(void *instance, REQUEST *request,
+		    char *fmt, char *out, int freespace,
+		    RADIUS_ESCAPE_STRING func)
 {
 	SQLSOCK *sqlsocket;
 	SQL_ROW row;
 	SQL_INST *inst = instance;
 	char querystr[MAX_QUERY_LEN];
+	char sqlusername[2 * MAX_STRING_LEN + 10];
 	int ret = 0;
 
 	DEBUG("rlm_sql (%s): - sql_xlat", inst->config->xlat_name);
-
+	/*
+         * Add SQL-User-Name attribute just in case it is needed
+         *  We could search the string fmt for SQL-User-Name to see if this is
+         *  needed or not
+         */
+	sql_set_user(inst, request, sqlusername, NULL);
 	/*
 	 * Do an xlat on the provided string (nice recursive operation).
 	 */
@@ -128,12 +176,14 @@ static int sql_xlat(void *instance, REQUEST *request, char *fmt, char *out, int
 		return 0;
 	}
 
+	query_log(request, inst,querystr);
 	sqlsocket = sql_get_socket(inst);
 	if (sqlsocket == NULL)
 		return 0;
 	if (rlm_sql_select_query(sqlsocket,inst,querystr)){
-		radlog(L_ERR, "rlm_sql (%s): database query error",
-		       inst->config->xlat_name);
+		radlog(L_ERR, "rlm_sql (%s): database query error, %s: %s",
+		       inst->config->xlat_name,querystr,
+		       (char *)(inst->module->sql_error)(sqlsocket, inst->config));
 		sql_release_socket(inst,sqlsocket);
 		return 0;
 	}
@@ -228,7 +278,8 @@ static int sql_escape_func(char *out, int outlen, const char *in)
 /*
  *	Set the SQl user name.
  */
-static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, const char *username) {
+static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, const char *username)
+{
 	VALUE_PAIR *vp=NULL;
 	char tmpuser[MAX_STRING_LEN];
 
@@ -247,7 +298,7 @@ static int sql_set_user(SQL_INST *inst, REQUEST *request, char *sqlusername, con
 	}
 
 	if (*tmpuser) {
-		strNcpy(sqlusername, tmpuser, MAX_STRING_LEN * 2);
+		strNcpy(sqlusername, tmpuser, sizeof(tmpuser));
 		DEBUG2("rlm_sql (%s): sql_set_user escaped user --> '%s'",
 		       inst->config->xlat_name, sqlusername);
 		vp = pairmake("SQL-User-Name", sqlusername, 0);
@@ -313,8 +364,9 @@ static int sql_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE
 	if (sqlsocket == NULL)
 		return 1;
 	if ((inst->module->sql_select_query)(sqlsocket,inst->config,querystr) <0){
-		radlog(L_ERR, "rlm_sql (%s): database query error",
-		       inst->config->xlat_name);
+		radlog(L_ERR, "rlm_sql (%s): database query error, %s: %s",
+		       inst->config->xlat_name,querystr,
+		       (char *)(inst->module->sql_error)(sqlsocket,inst->config));
 		sql_release_socket(inst,sqlsocket);
 		return 1;
 	}
@@ -333,8 +385,7 @@ static int sql_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE
 			DEBUG("rlm_sql (%s): - sql_groupcmp finished: User belongs in group %s",
 			      inst->config->xlat_name,
 			      (char *)check->strvalue);
-			(inst->module->sql_finish_select_query)(sqlsocket, inst->config);
-			sql_release_socket(inst, sqlsocket);
+			(inst->module->sql_finish_select_query)(sqlsocket, inst->config);			sql_release_socket(inst, sqlsocket);
 			return 0;
 		}
 	}
@@ -349,10 +400,65 @@ static int sql_groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE
 }
 
 
-static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
+static int rlm_sql_detach(void *instance)
+{
+	SQL_INST *inst = instance;
 
+	if (inst->sqlpool) {
+		sql_poolfree(inst);
+	}
+
+	if (inst->config->xlat_name) {
+		xlat_unregister(inst->config->xlat_name,sql_xlat);
+		free(inst->config->xlat_name);
+	}
+
+	paircompare_unregister(PW_SQL_GROUP, sql_groupcmp);
+
+	if (inst->config) {
+		int i;
+
+		/*
+		 *	Free up dynamically allocated string pointers.
+		 */
+		for (i = 0; module_config[i].name != NULL; i++) {
+			char **p;
+			if (module_config[i].type != PW_TYPE_STRING_PTR) {
+				continue;
+			}
+			
+			/*
+			 *	Treat 'config' as an opaque array of bytes,
+			 *	and take the offset into it.  There's a
+			 *      (char*) pointer at that offset, and we want
+			 *	to point to it.
+			 */
+			p = (char **) (((char *)inst->config) + module_config[i].offset);
+			if (!*p) { /* nothing allocated */
+				continue;
+			}
+			free(*p);
+			*p = NULL;
+		}
+		free(inst->config);
+		inst->config = NULL;
+	}
+
+	if (inst->handle) {
+#if 0
+		/*
+		 *	FIXME: Call the modules 'destroy' function?
+		 */
+		lt_dlclose(inst->handle);	/* ignore any errors */
+#endif
+	}
+	free(inst);
+
+	return 0;
+}
+static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance)
+{
 	SQL_INST *inst;
-	lt_dlhandle handle;
 	char *xlat_name;
 
 	inst = rad_malloc(sizeof(SQL_INST));
@@ -366,8 +472,7 @@ static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
 	 * fail.
 	 */
 	if (cf_section_parse(conf, inst->config, module_config) < 0) {
-		free(inst->config);
-		free(inst);
+		rlm_sql_detach(inst);
 		return -1;
 	}
 
@@ -382,26 +487,37 @@ static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
 	if (inst->config->num_sql_socks > MAX_SQL_SOCKS) {
 		radlog(L_ERR | L_CONS, "rlm_sql (%s): sql_instantiate: number of sqlsockets cannot exceed MAX_SQL_SOCKS, %d",
 		       inst->config->xlat_name, MAX_SQL_SOCKS);
-		free(inst->config);
-		free(inst);
+		rlm_sql_detach(inst);
 		return -1;
 	}
 
-	handle = lt_dlopenext(inst->config->sql_driver);
-	if (handle == NULL) {
+	/*
+	 *	Sanity check for crazy people.
+	 */
+	if (strncmp(inst->config->sql_driver, "rlm_sql_", 8) != 0) {
+		radlog(L_ERR, "rlm_sql (%s): \"%s\" is NOT an SQL driver!",
+		       inst->config->xlat_name, inst->config->sql_driver);
+		rlm_sql_detach(inst);
+		return -1;
+	}
+
+	inst->handle = lt_dlopenext(inst->config->sql_driver);
+	if (inst->handle == NULL) {
 		radlog(L_ERR, "rlm_sql (%s): Could not link driver %s: %s",
 		       inst->config->xlat_name, inst->config->sql_driver,
 		       lt_dlerror());
 		radlog(L_ERR, "rlm_sql (%s): Make sure it (and all its dependent libraries!) are in the search path of your system's ld.",
 		       inst->config->xlat_name);
+		rlm_sql_detach(inst);
 		return -1;
 	}
 
-	inst->module = (rlm_sql_module_t *) lt_dlsym(handle, inst->config->sql_driver);
+	inst->module = (rlm_sql_module_t *) lt_dlsym(inst->handle, inst->config->sql_driver);
 	if (!inst->module) {
 		radlog(L_ERR, "rlm_sql (%s): Could not link symbol %s: %s",
 		       inst->config->xlat_name, inst->config->sql_driver,
 		       lt_dlerror());
+		rlm_sql_detach(inst);
 		return -1;
 	}
 
@@ -414,8 +530,7 @@ static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
 	       inst->config->sql_db);
 
 	if (sql_init_socketpool(inst) < 0) {
-		free(inst->config);
-		free(inst);
+		rlm_sql_detach(inst);
 		return -1;
 	}
 	paircompare_register(PW_SQL_GROUP, PW_USER_NAME, sql_groupcmp, inst);
@@ -425,28 +540,14 @@ static int rlm_sql_instantiate(CONF_SECTION * conf, void **instance) {
 	return RLM_MODULE_OK;
 }
 
-static int rlm_sql_destroy(void) {
-
-	return 0;
-}
-
-static int rlm_sql_detach(void *instance) {
-
-	SQL_INST *inst = instance;
-
-	sql_poolfree(inst);
-	if (inst->config->xlat_name)
-		xlat_unregister(inst->config->xlat_name,sql_xlat);
-	paircompare_unregister(PW_SQL_GROUP, sql_groupcmp);
-	free(inst->config);
-	free(inst);
-
+static int rlm_sql_destroy(void)
+{
 	return 0;
 }
 
 
-static int rlm_sql_authorize(void *instance, REQUEST * request) {
-
+static int rlm_sql_authorize(void *instance, REQUEST * request)
+{
 	VALUE_PAIR *check_tmp = NULL;
 	VALUE_PAIR *reply_tmp = NULL;
 	VALUE_PAIR *user_profile = NULL;
@@ -573,7 +674,7 @@ static int rlm_sql_authorize(void *instance, REQUEST * request) {
 	*/
 
 	if (paircmp(request, request->packet->vps, check_tmp, &reply_tmp) != 0) {
-		radlog(L_INFO, "rlm_sql (%s): Pairs do not match for user [%s]",
+		radlog(L_INFO, "rlm_sql (%s): No matching entry in the database for request from user [%s]",
 		       inst->config->xlat_name, sqlusername);
 		/* Remove the username we (maybe) added above */
 		pairdelete(&request->packet->vps, PW_SQL_USER_NAME);
@@ -608,7 +709,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 	int     acctstatustype = 0;
 	char    querystr[MAX_QUERY_LEN];
 	char    logstr[MAX_QUERY_LEN];
-	char	sqlusername[MAX_STRING_LEN];
+	char	sqlusername[2 * MAX_STRING_LEN + 10];
 
 #ifdef CISCO_ACCOUNTING_HACK
 	int     acctsessiontime = 0;
@@ -636,7 +737,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 		case PW_STATUS_ACCOUNTING_OFF:
 			radlog(L_INFO, "rlm_sql (%s): received Acct On/Off packet", inst->config->xlat_name);
 			radius_xlat(querystr, sizeof(querystr), inst->config->accounting_onoff_query, request, sql_escape_func);
-			query_log(inst, querystr);
+			query_log(request, inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
@@ -664,7 +765,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			sql_set_user(inst, request, sqlusername, NULL);
 
 			radius_xlat(querystr, sizeof(querystr), inst->config->accounting_update_query, request, sql_escape_func);
-			query_log(inst, querystr);
+			query_log(request, inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
@@ -692,7 +793,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			sql_set_user(inst, request, sqlusername, NULL);
 
 			radius_xlat(querystr, sizeof(querystr), inst->config->accounting_start_query, request, sql_escape_func);
-			query_log(inst, querystr);
+			query_log(request, inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
@@ -709,7 +810,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 					 * our alternate query now (typically an UPDATE)
 					 */
 					radius_xlat(querystr, sizeof(querystr), inst->config->accounting_start_query_alt, request, sql_escape_func);
-					query_log(inst, querystr);
+					query_log(request, inst, querystr);
 
 					if (*querystr) { /* non-empty query */
 						if (rlm_sql_query(sqlsocket, inst, querystr)) {
@@ -736,7 +837,7 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 			sql_set_user(inst, request, sqlusername, NULL);
 
 			radius_xlat(querystr, sizeof(querystr), inst->config->accounting_stop_query, request, sql_escape_func);
-			query_log(inst, querystr);
+			query_log(request, inst, querystr);
 
 			sqlsocket = sql_get_socket(inst);
 			if (sqlsocket == NULL)
@@ -771,12 +872,12 @@ static int rlm_sql_accounting(void *instance, REQUEST * request) {
 				        	        radius_xlat(logstr, sizeof(logstr), "rlm_sql: Stop packet with zero session length.  (user '%{User-Name}', nas '%{NAS-IP-Address}')", request, sql_escape_func);
 					                radlog(L_ERR, logstr);
 				        	        sql_release_socket(inst, sqlsocket);
-				                	return RLM_MODULE_NOOP;
+				                	ret = RLM_MODULE_NOOP;
 					        }
 #endif
 
 						radius_xlat(querystr, sizeof(querystr), inst->config->accounting_stop_query_alt, request, sql_escape_func);
-						query_log(inst, querystr);
+						query_log(request, inst, querystr);
 
 						if (*querystr) { /* non-empty query */
 							if (rlm_sql_query(sqlsocket, inst, querystr)) {
