@@ -2,7 +2,7 @@
  *  sql.c		rlm_sql - FreeRADIUS SQL Module
  *		Main code directly taken from ICRADIUS
  *
- * Version:	$Id: sql.c,v 1.58.2.2 2002/12/06 18:00:25 aland Exp $
+ * Version:	$Id: sql.c,v 1.72.2.4 2003/08/27 16:00:43 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -44,14 +44,14 @@
 #include	<errno.h>
 #include	<sys/wait.h>
 
-#if HAVE_PTHREAD_H
-#include	<pthread.h>
-#endif
-
 #include	"radiusd.h"
 #include	"conffile.h"
 #include	"rlm_sql.h"
 
+#if HAVE_PTHREAD_H
+#include	<pthread.h>
+#endif
+
 
 /*
  * Connect to a server.  If error, set this socket's state to be
@@ -61,21 +61,27 @@
  * successful in connecting, set state to sockconnected.
  * - chad
  */
-static int connect_single_socket(SQLSOCK *sqlsocket, SQL_INST *inst) {
+static int connect_single_socket(SQLSOCK *sqlsocket, SQL_INST *inst)
+{
+	int rcode;
 	radlog(L_DBG, "rlm_sql (%s): Attempting to connect %s #%d",
 	       inst->config->xlat_name, inst->module->name, sqlsocket->id);
 
-	if ((inst->module->sql_init_socket)(sqlsocket, inst->config) < 0) {
-		radlog(L_CONS | L_ERR, "rlm_sql (%s): Failed to connect DB handle #%d", inst->config->xlat_name, sqlsocket->id);
-		inst->connect_after = time(NULL) + inst->config->connect_failure_retry_delay;
-		sqlsocket->state = sockunconnected;
-		return(-1);
-	} else {
+	rcode = (inst->module->sql_init_socket)(sqlsocket, inst->config);
+	if (rcode == 0) {
 		radlog(L_DBG, "rlm_sql (%s): Connected new DB handle, #%d",
 		       inst->config->xlat_name, sqlsocket->id);
 		sqlsocket->state = sockconnected;
 		return(0);
 	}
+
+	/*
+	 *  Error, or SQL_DOWN.
+	 */
+	radlog(L_CONS | L_ERR, "rlm_sql (%s): Failed to connect DB handle #%d", inst->config->xlat_name, sqlsocket->id);
+	inst->connect_after = time(NULL) + inst->config->connect_failure_retry_delay;
+	sqlsocket->state = sockunconnected;
+	return(-1);
 }
 
 
@@ -86,15 +92,14 @@ static int connect_single_socket(SQLSOCK *sqlsocket, SQL_INST *inst) {
  *	Purpose: Connect to the sql server, if possible
  *
  *************************************************************************/
-int sql_init_socketpool(SQL_INST * inst) {
-
+int sql_init_socketpool(SQL_INST * inst)
+{
+	int i, rcode;
+	int success = 0;
 	SQLSOCK *sqlsocket;
-	int     i;
 
 	inst->connect_after = 0;
-	inst->used = 0;
 	inst->sqlpool = NULL;
-	inst->socknr = 0;
 
 	for (i = 0; i < inst->config->num_sql_socks; i++) {
 		radlog(L_DBG, "rlm_sql (%s): starting %d",
@@ -108,30 +113,35 @@ int sql_init_socketpool(SQL_INST * inst) {
 		sqlsocket->id = i;
 		sqlsocket->state = sockunconnected;
 
-#if HAVE_SEMAPHORE_H
-		/*
-		 *  FIXME! Check return codes!
-		 */
-		sqlsocket->semaphore = (sem_t *) rad_malloc(sizeof(sem_t));
-		sem_init(sqlsocket->semaphore, 0, SQLSOCK_UNLOCKED);
-#else
-		sqlsocket->in_use = SQLSOCK_UNLOCKED;
+#if HAVE_PTHREAD_H
+		rcode = pthread_mutex_init(&sqlsocket->mutex,NULL);
+		if (rcode != 0) {
+			radlog(L_ERR, "rlm_sql: Failed to init lock: %s",
+			       strerror(errno));
+			return 0;
+		}
 #endif
 
 		if (time(NULL) > inst->connect_after) {
-			/* this sets the sqlsocket->state, and possibly sets inst->connect_after */
-			/* FIXME! check return code */
-			connect_single_socket(sqlsocket, inst);
+			/*
+			 *	This sets the sqlsocket->state, and
+			 *	possibly also inst->connect_after
+			 */
+			if (connect_single_socket(sqlsocket, inst) == 0) {
+				success = 1;
+			}
 		}
 
 		/* Add this socket to the list of sockets */
 		sqlsocket->next = inst->sqlpool;
 		inst->sqlpool = sqlsocket;
 	}
+	inst->last_used = NULL;
 
-#if HAVE_PTHREAD_H
-	pthread_mutex_init(&inst->mutex, NULL);
-#endif
+	if (!success) {
+		radlog(L_DBG, "rlm_sql (%s): Failed to connect to any SQL server.",
+		       inst->config->xlat_name);
+	}
 
 	return 1;
 }
@@ -143,16 +153,17 @@ int sql_init_socketpool(SQL_INST * inst) {
  *     Purpose: Clean up and free sql pool
  *
  *************************************************************************/
-void sql_poolfree(SQL_INST * inst) {
-
+void sql_poolfree(SQL_INST * inst)
+{
 	SQLSOCK *cur;
+	SQLSOCK *next;
 
-	for (cur = inst->sqlpool; cur; cur = cur->next) {
+	for (cur = inst->sqlpool; cur; cur = next) {
+		next = cur->next;
 		sql_close_socket(inst, cur);
 	}
-#if HAVE_PTHREAD_H
-	pthread_mutex_destroy(&inst->mutex);
-#endif
+
+	inst->sqlpool = NULL;
 }
 
 
@@ -163,13 +174,18 @@ void sql_poolfree(SQL_INST * inst) {
  *	Purpose: Close and free a sql sqlsocket
  *
  *************************************************************************/
-int sql_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket) {
-
+int sql_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket)
+{
 	radlog(L_DBG, "rlm_sql (%s): Closing sqlsocket %d",
 	       inst->config->xlat_name, sqlsocket->id);
-	(inst->module->sql_close)(sqlsocket, inst->config);
-#if HAVE_SEMAPHORE_H
-	sem_destroy(sqlsocket->semaphore);
+	if (sqlsocket->state == sockconnected) {
+		(inst->module->sql_close)(sqlsocket, inst->config);
+	}
+	if (inst->module->sql_destroy_socket) {
+		(inst->module->sql_destroy_socket)(sqlsocket, inst->config);
+	}
+#if HAVE_PTHREAD_H
+	pthread_mutex_destroy(&sqlsocket->mutex);
 #endif
 	free(sqlsocket);
 	return 1;
@@ -183,70 +199,101 @@ int sql_close_socket(SQL_INST *inst, SQLSOCK * sqlsocket) {
  *	Purpose: Return a SQL sqlsocket from the connection pool           
  *
  *************************************************************************/
-SQLSOCK * sql_get_socket(SQL_INST * inst) {
-	SQLSOCK *cur, *cur2;
+SQLSOCK * sql_get_socket(SQL_INST * inst)
+{
+	SQLSOCK *cur, *start;
 	int tried_to_connect = 0;
-
-	while (inst->used == inst->config->num_sql_socks) {
-		radlog(L_ERR, "rlm_sql (%s): All sockets are being used! Please increase the maximum number of sockets!", inst->config->xlat_name);
-		return NULL;
-	}
+	int unconnected = 0;
 
 	/*
-	 * Rotating the socket so that all get used and none get closed due to
-	 * inactivity from the SQL server ( such as mySQL ).
+	 *	Start at the last place we left off.
 	 */
+	start = inst->last_used;
+	if (!start) start = inst->sqlpool;
+
+	cur = start;
+
+	while (cur) {
 #if HAVE_PTHREAD_H
-	pthread_mutex_lock(&inst->mutex);
+		/*
+		 *	If this socket is in use by another thread,
+		 *	skip it, and try another socket.
+		 *
+		 *	If it isn't used, then grab it ourselves.
+		 */
+		if (pthread_mutex_trylock(&cur->mutex) != 0) {
+			goto next;
+		} /* else we now have the lock */
 #endif
 
-	if(inst->socknr == 0) {
-	        inst->socknr = inst->config->num_sql_socks;
-	}
-	inst->socknr--;
-	cur2 = inst->sqlpool;
-	while (inst->socknr != cur2->id) {
-	        cur2 = cur2->next;
-	}
-#if HAVE_PTHREAD_H
-	pthread_mutex_unlock(&inst->mutex);
-#endif
-
-	for (cur = cur2; cur; cur = cur->next) {
-
-		/* if we happen upon an unconnected socket, and this instance's grace 
-		 * period on (re)connecting has expired, then try to connect it.  This 
-		 * should be really rare.  - chad
+		/*
+		 *	If we happen upon an unconnected socket, and
+		 *	this instance's grace period on
+		 *	(re)connecting has expired, then try to
+		 *	connect it.  This should be really rare.
 		 */
 		if ((cur->state == sockunconnected) && (time(NULL) > inst->connect_after)) {
-			tried_to_connect = 1;
-			radlog(L_INFO, "rlm_sql (%s): Trying to (re)connect an unconnected handle...", inst->config->xlat_name);
+			radlog(L_INFO, "rlm_sql (%s): Trying to (re)connect unconnected handle %d..", inst->config->xlat_name, cur->id);
+			tried_to_connect++;
 			connect_single_socket(cur, inst);
 		}
 
 		/* if we still aren't connected, ignore this handle */
 		if (cur->state == sockunconnected) {
-			radlog(L_DBG, "rlm_sql (%s): Ignoring unconnected handle", inst->config->xlat_name);
-			continue;
+			radlog(L_DBG, "rlm_sql (%s): Ignoring unconnected handle %d..", inst->config->xlat_name, cur->id);
+		        unconnected++;
+#if HAVE_PTHREAD_H
+			pthread_mutex_unlock(&cur->mutex);
+#endif
+			goto next;
 		}
 
-#if HAVE_SEMAPHORE_H
-		if (sem_trywait(cur->semaphore) == 0) {
-#else
-		if (cur->in_use == SQLSOCK_UNLOCKED) {
-#endif
-			(inst->used)++;
-#ifndef HAVE_SEMAPHORE_H
-			cur->in_use = SQLSOCK_LOCKED;
-#endif
-			radlog(L_DBG, "rlm_sql (%s): Reserving sql socket id: %d",
-			       inst->config->xlat_name, cur->id);
-			return cur;
+		/* should be connected, grab it */
+		radlog(L_DBG, "rlm_sql (%s): Reserving sql socket id: %d", inst->config->xlat_name, cur->id);
+
+		if (unconnected != 0 || tried_to_connect != 0) {
+			radlog(L_INFO, "rlm_sql (%s): got socket %d after skipping %d unconnected handles, tried to reconnect %d though", inst->config->xlat_name, cur->id, unconnected, tried_to_connect);
+		}
+		
+		/*
+		 *	The socket is returned in the locked
+		 *	state.
+		 *
+		 *	We also remember where we left off,
+		 *	so that the next search can start from
+		 *	here.
+		 *
+		 *	Note that multiple threads MAY over-write
+		 *	the 'inst->last_used' variable.  This is OK,
+		 *	as it's a pointer only used for reading.
+		 */
+		inst->last_used = cur->next;
+		return cur;
+
+		/* move along the list */
+	next:
+		cur = cur->next;
+
+		/*
+		 *	Because we didnt start at the start, once we
+		 *	hit the end of the linklist, we should go
+		 *	back to the beginning and work toward the
+		 *	middle!
+		 */
+		if (!cur) {
+			cur = inst->sqlpool;
+		}
+
+		/*
+		 *	If we're at the socket we started 
+		 */
+		if (cur == start) {
+			break;
 		}
 	}
 
 	/* We get here if every DB handle is unconnected and unconnectABLE */
-	radlog((tried_to_connect == 0) ? (L_DBG) : (L_CONS | L_ERR), "rlm_sql (%s): There are no DB handles to use!", inst->config->xlat_name);
+	radlog(L_INFO, "rlm_sql (%s): There are no DB handles to use! skipped %d, tried to connect %d", inst->config->xlat_name, unconnected, tried_to_connect);
 	return NULL;
 }
 
@@ -257,13 +304,10 @@ SQLSOCK * sql_get_socket(SQL_INST * inst) {
  *	Purpose: Frees a SQL sqlsocket back to the connection pool           
  *
  *************************************************************************/
-int sql_release_socket(SQL_INST * inst, SQLSOCK * sqlsocket) {
-
-	(inst->used)--;
-#if HAVE_SEMAPHORE_H
-	sem_post(sqlsocket->semaphore);
-#else
-	sqlsocket->in_use = SQLSOCK_UNLOCKED;
+int sql_release_socket(SQL_INST * inst, SQLSOCK * sqlsocket)
+{
+#if HAVE_PTHREAD_H
+	pthread_mutex_unlock(&sqlsocket->mutex);
 #endif
 
 	radlog(L_DBG, "rlm_sql (%s): Released sql socket id: %d",
@@ -280,8 +324,8 @@ int sql_release_socket(SQL_INST * inst, SQLSOCK * sqlsocket) {
  *	Purpose: Read entries from the database and fill VALUE_PAIR structures
  *
  *************************************************************************/
-int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int querymode) {
-
+int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int querymode)
+{
 	DICT_ATTR *attr;
 	VALUE_PAIR *pair, *check;
 	char *ptr;
@@ -332,14 +376,21 @@ int sql_userparse(VALUE_PAIR ** first_pair, SQL_ROW row, int querymode) {
  *	Purpose: call the module's sql_fetch_row and implement re-connect
  *
  *************************************************************************/
-int rlm_sql_fetch_row(SQLSOCK *sqlsocket, SQL_INST *inst) {
+int rlm_sql_fetch_row(SQLSOCK *sqlsocket, SQL_INST *inst)
+{
 	int ret;
 
-	ret = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
+	if (sqlsocket->conn) {
+		ret = (inst->module->sql_fetch_row)(sqlsocket, inst->config);
+	} else {
+		ret = SQL_DOWN;
+	}
 
 	if (ret == SQL_DOWN) {
-	        /* close the socket that failed */
-	        (inst->module->sql_close)(sqlsocket, inst->config);
+	        /* close the socket that failed, but only if it was open */
+		if (sqlsocket->conn) {
+			(inst->module->sql_close)(sqlsocket, inst->config);
+		}
 
 		/* reconnect the socket */
 		if (connect_single_socket(sqlsocket, inst) < 0) {
@@ -367,7 +418,8 @@ int rlm_sql_fetch_row(SQLSOCK *sqlsocket, SQL_INST *inst) {
  *	Purpose: call the module's sql_query and implement re-connect
  *
  *************************************************************************/
-int rlm_sql_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
+int rlm_sql_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query)
+{
 	int ret;
 
 	/*
@@ -409,7 +461,8 @@ int rlm_sql_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
  *	Purpose: call the module's sql_select_query and implement re-connect
  *
  *************************************************************************/
-int rlm_sql_select_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
+int rlm_sql_select_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query)
+{
 	int ret;
 
 	/*
@@ -452,8 +505,8 @@ int rlm_sql_select_query(SQLSOCK *sqlsocket, SQL_INST *inst, char *query) {
  *	Purpose: Get any group check or reply pairs
  *
  *************************************************************************/
-int sql_getvpdata(SQL_INST * inst, SQLSOCK * sqlsocket, VALUE_PAIR **pair, char *query, int mode) {
-
+int sql_getvpdata(SQL_INST * inst, SQLSOCK * sqlsocket, VALUE_PAIR **pair, char *query, int mode)
+{
 	SQL_ROW row;
 	int     rows = 0;
 
@@ -484,14 +537,24 @@ int sql_getvpdata(SQL_INST * inst, SQLSOCK * sqlsocket, VALUE_PAIR **pair, char
 	return rows;
 }
 
-void query_log(SQL_INST * inst, char *querystr) {
+void query_log(REQUEST *request, SQL_INST *inst, char *querystr)
+{
 	FILE   *sqlfile = NULL;
 
 	if (inst->config->sqltrace) {
-		if ((sqlfile = fopen(inst->config->tracefile, "a")) == (FILE *) NULL) {
+		char buffer[8192];
+
+		if (!radius_xlat(buffer, sizeof(buffer),
+				 inst->config->tracefile, request, NULL)) {
+		  radlog(L_ERR, "rlm_sql (%s): xlat failed.",
+			 inst->config->xlat_name);
+		  return;
+		}
+
+		if ((sqlfile = fopen(buffer, "a")) == (FILE *) NULL) {
 			radlog(L_ERR, "rlm_sql (%s): Couldn't open file %s",
 			       inst->config->xlat_name,
-			       inst->config->tracefile);
+			       buffer);
 		} else {
 			int fd = fileno(sqlfile);
 			
