@@ -3,7 +3,7 @@
  *		accounting:     Functions to write radwtmp file.
  *		Also contains handler for "Group".
  *
- * Version:	$Id: rlm_unix.c,v 1.59 2003/07/07 19:11:45 aland Exp $
+ * Version:	$Id: rlm_unix.c,v 1.59.2.2 2003/08/26 12:37:50 phampson Exp $
  *
  *   This program is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU General Public License as published by
@@ -23,7 +23,7 @@
  * Copyright 2000  Jeff Carneal <jeff@apex.net>
  * Copyright 2000  Alan Curry <pacman@world.std.com>
  */
-static const char rcsid[] = "$Id: rlm_unix.c,v 1.59 2003/07/07 19:11:45 aland Exp $";
+static const char rcsid[] = "$Id: rlm_unix.c,v 1.59.2.2 2003/08/26 12:37:50 phampson Exp $";
 
 #include	"autoconf.h"
 #include	"libradius.h"
@@ -209,6 +209,12 @@ static int groupcmp(void *instance, REQUEST *req, VALUE_PAIR *request, VALUE_PAI
 		return 1;
 	}
 
+	/*
+	 *	No user name, doesn't compare.
+	 */
+	if (!req->username) {
+		return -1;
+	}
 	username = (char *)req->username->strvalue;
 
 	if (group_inst->cache_passwd &&
